﻿<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/default_dune_plugin_fw.php';
require_once 'lib/default_dune_plugin.php';
require_once 'lib/utils.php';
require_once 'lib/control_factory.php';
require_once 'lib/action_factory.php';
require_once 'lib/list_util.php';

require_once 'utils.php';

require 'cursor.php';
require 'folder.php';
require 'folder_cache.php';
require 'playback_url_cache.php';
require 'video_cache.php';
require 'youtube_api.php';
require 'vimeo_api.php';

///////////////////////////////////////////////////////////////////////////

$hd_interface_language = 'english';

///////////////////////////////////////////////////////////////////////////

function update_interface_language()
{
    global $hd_interface_language;

    $v =
        trim(
            shell_exec(
                "grep 'interface_language' /config/settings.properties" . ' | sed "s/^.*= *//"'));

    $lang_saved = $hd_interface_language;
    $hd_interface_language = (isset($v) && strlen($v) > 0) ? $v : 'english';

    return ($lang_saved !== $hd_interface_language) ? 1 : 0;
}

///////////////////////////////////////////////////////////////////////////

function get_interface_language()
{
    global $hd_interface_language;
    return $hd_interface_language;
}

function is_interface_language_russian_or_similar()
{
    return get_interface_language() === 'russian' ||
        get_interface_language() === 'ukrainian';
}

///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////

class cScVODPlugin extends DefaultDunePlugin implements UserInputHandler
{
    const ID = 'cScVOD';
    ///////////////////////////////////////////////////////////////////////
    
    private $fav_build_in_progess;

    private static $open_folder_action = array
    (
        GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
    );

    ///////////////////////////////////////////////////////////////////////

    private static $vod_play_action = array
    (
        GuiAction::handler_string_id => PLUGIN_VOD_PLAY_ACTION_ID,
        GuiAction::caption           => 'Play'
    );

    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////

    private static $catalog_vip = array
    (
        ViewItemParams::icon_path => 'gui_skin://small_icons/folder.aai',
        ViewItemParams::item_layout => HALIGN_LEFT,
        ViewItemParams::icon_valign => VALIGN_CENTER,
        ViewItemParams::icon_dx => 20,
        ViewItemParams::icon_dy => -5,
        ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
        ViewItemParams::item_caption_width => 1550
    );

    public static function get_skin_path($plugin_cookies)
    {
        $skin = isset($plugin_cookies->skin) ? $plugin_cookies->skin : 'default';
        
        $skins = array
        (
            'default' =>  'plugin_file://skins/default/',
            'juniperus' => 'plugin_file://skins/juniperus/'
        );
        
        return $skins[$skin];
    }

    public static function get_main_vip($plugin_cookies)
    {
        $skin = isset($plugin_cookies->skin) ? $plugin_cookies->skin : 'default';
        
        $skins = array
        (
            'default' =>  array
                            (
                                ViewItemParams::icon_scale_factor =>0.75,
                                ViewItemParams::icon_sel_scale_factor =>1,
                                ViewItemParams::icon_path => 'gui_skin://small_icons/folder.aai',
                                ViewItemParams::item_layout => VALIGN_CENTER,
                                ViewItemParams::icon_valign => VALIGN_CENTER,
                                ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL
                            ),
            'juniperus' =>    array
                            (
                                ViewItemParams::icon_scale_factor =>1.0,        //jun
                                ViewItemParams::icon_sel_scale_factor =>1.2,        //jun
                                ViewItemParams::icon_path => 'gui_skin://small_icons/folder.aai',
                                ViewItemParams::item_layout => VALIGN_CENTER,
                                ViewItemParams::icon_valign => VALIGN_CENTER,
                                ViewItemParams::item_caption_dy => -50,                     //jun
                                ViewItemParams::item_caption_sel_dy => -15,                //jun
                                ViewItemParams::icon_dy => -14,                            //jun
                                ViewItemParams::icon_sel_dy => -14,                         //jun
                                ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL
                            )
        );
        
        return $skins[$skin];
    }

    public static function get_main_menu_folder_view($plugin_cookies)
    {
        $skin = isset($plugin_cookies->skin) ? $plugin_cookies->skin : 'default';
        
        $skins = array
        (
            'juniperus' =>  array
                            (
                                ViewParams::num_cols => 4,        //jun
                                ViewParams::num_rows => 2,        //jun
                                ViewParams::icon_selection_box_dx => -118,        //jun
                                ViewParams::icon_selection_box_dy => -154,        //jun
                                ViewParams::icon_selection_box_width => 240,        //jun
                                ViewParams::icon_selection_box_height => 240,        //jun
                                ViewParams::paint_sandwich => false,                    //jun
                                ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                                ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                                ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                                ViewParams::sandwich_width => 200,            //jun
                                ViewParams::sandwich_height => 200,         //jun
                                ViewParams::sandwich_icon_upscale_enabled => false,        //jun
                                ViewParams::paint_details => false        //jun
                            ),
            'default' =>    array
                            (
                                ViewParams::num_cols => 3,
                                ViewParams::num_rows => 1,
                                ViewParams::icon_selection_box_dx => -130,
                                ViewParams::icon_selection_box_dy => -130,
                                ViewParams::icon_selection_box_width => 360,
                                ViewParams::icon_selection_box_height => 200,
                                ViewParams::paint_sandwich => true,
                                ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                                ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                                ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                                ViewParams::sandwich_width => 360,
                                ViewParams::sandwich_height => 200,
                                ViewParams::sandwich_icon_upscale_enabled => true,
                                ViewParams::paint_details => true
                            ),
        );
        
        return $skins[$skin];
    }

    public function __construct()
    {
        $this->fav_build_in_progess = false;
    }

    private static function get_preview_url($v)
    {
        if (!HD::has_attribute($v, 'season_image'))
            return '';

        return $v->season_image;
    }

    private function get_quality_array($cfg)
    {
        // 240, 360, 480, 720 и 1080
        $vquality = array();
        if ($cfg === true)
            $vquality[0] = '240 (Минимально возможное)';
        else
            $vquality[0] = '240';
        $vquality[1] = '360';
        $vquality[2] = '480';
        $vquality[3] = '720';
        if ($cfg === true)
            $vquality[4] = '1080 (Максимально возможное)';
        else
            $vquality[4] = '1080';

        return $vquality;        
    }
    ///////////////////////////////////////////////////////////////////////
    public function get_play_action($media_url, &$plugin_cookies)
    {	//-//hd_print('--->>>1 --->>>' . $media_url);
        $add_play_action = UserInputHandlerRegistry::create_action(
                        $this, 'play_item');
        $add_play_action['caption'] = (
                is_interface_language_russian_or_similar() ? 'Воспроизвести' : "Play"
                );

        return $add_play_action;
    }

    ///////////////////////////////////////////////////////////////////////
    public function get_item_action_map($media_url, &$plugin_cookies)
    {	//-//hd_print('--->>>2 --->>>' . $media_url);
        $add_fav_action = UserInputHandlerRegistry::create_action(
                        $this, 'add_favorite');
        $add_fav_action['caption'] = (
                is_interface_language_russian_or_similar() ? 'Добавить в Избранное' : "Add Favorite"
                );

        $switch_view = UserInputHandlerRegistry::create_action(
                        $this, 'switch_view');
        $switch_view['caption'] = (
                is_interface_language_russian_or_similar() ? "Вид" : 'View mode'
                );
        $view_info = UserInputHandlerRegistry::create_action(
                        $this, 'ex_view_info');
        $view_info['caption'] = (
                is_interface_language_russian_or_similar() ? "Инфо" : 'Info'
                );
		$search_view = UserInputHandlerRegistry::create_action(
                        $this, 'do_search_menu');
        $search_view['caption'] = (is_interface_language_russian_or_similar() ? "Поиск" : 'Search');
		
        $skip_details = isset($plugin_cookies->skip_details) ? $plugin_cookies->skip_details : 'no';

        if ($skip_details === 'yes')
        {
            return array
                (
                GUI_EVENT_KEY_ENTER => UserInputHandlerRegistry::create_action($this, 'view_series'),
                GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
                GUI_EVENT_KEY_A_RED => $switch_view,
                GUI_EVENT_KEY_INFO => $view_info,
				GUI_EVENT_KEY_B_GREEN => $search_view,
				GUI_EVENT_KEY_D_BLUE => $view_info,
                GUI_EVENT_KEY_C_YELLOW => $add_fav_action
            );
            
        }
        else
        {
            return array
                (
                GUI_EVENT_KEY_ENTER => $view_info,
                GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
                GUI_EVENT_KEY_A_RED => $switch_view,
				GUI_EVENT_KEY_B_GREEN => $search_view,
                GUI_EVENT_KEY_C_YELLOW => $add_fav_action
            );
            
        }
    }

    ///////////////////////////////////////////////////////////////////////
    public function get_search_action_map($media_url, &$plugin_cookies)
    {	//-//hd_print('--->>>3 --->>>' . $media_url);
        $perform_search_action = UserInputHandlerRegistry::create_action(
                        $this, 'perform_search');
        $perform_search_action['caption'] = (
                is_interface_language_russian_or_similar() ? 'Выполнить поиск' : "Perform Search"
                );
        $add_search_action = UserInputHandlerRegistry::create_action(
                        $this, 'add_search');
        $add_search_action['caption'] = (
                is_interface_language_russian_or_similar() ? 'Добавить поиск' : "Add Search"
                );
        $remove_search_action = UserInputHandlerRegistry::create_action(
                        $this, 'remove_search');
        $remove_search_action['caption'] = (
                is_interface_language_russian_or_similar() ? "Удалить" : 'Remove'
                );
        $move_search_top_action = UserInputHandlerRegistry::create_action(
                        $this, 'move_search_top');
        $move_search_top_action['caption'] = (
                is_interface_language_russian_or_similar() ? "Переместить вверх" : 'Move to top'
                );
        $menu_items[] = array(
            GuiMenuItemDef::caption => $move_search_top_action['caption'],
           GuiMenuItemDef::action => $move_search_top_action
        );
        $menu_items[] = array(
            GuiMenuItemDef::caption => $remove_search_action['caption'],
            GuiMenuItemDef::action => $remove_search_action
        );
        $popup_search_action = ActionFactory::show_popup_menu($menu_items);
        $do_search_action = UserInputHandlerRegistry::create_action($this, 'do_search');
        return array
            (
            GUI_EVENT_KEY_ENTER => $do_search_action,
            GUI_EVENT_KEY_POPUP_MENU => $popup_search_action
        );
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_fav_action_map($media_url, &$plugin_cookies)
    {	//-//hd_print('--->>>4 --->>>' . $media_url);
        $remove_fav_action = UserInputHandlerRegistry::create_action(
                        $this, 'remove_fav');
        $remove_fav_action['caption'] = (
                is_interface_language_russian_or_similar() ? "Удалить" : 'Remove'
                );
        $move_fav_top_action = UserInputHandlerRegistry::create_action(
                        $this, 'move_fav_top');
        $move_fav_top_action['caption'] = (
                is_interface_language_russian_or_similar() ? "Переместить вверх" : 'Move to top'
                );
        $switch_view = UserInputHandlerRegistry::create_action(
                        $this, 'switch_view');
        $switch_view['caption'] = (
                is_interface_language_russian_or_similar() ? "Вид" : 'View mode'
                );
        $view_info = UserInputHandlerRegistry::create_action(
                        $this, 'ex_view_info');
        $view_info['caption'] = (
                is_interface_language_russian_or_similar() ? "Инфо" : 'Info'
                );
       $menu_items[] = array(
            GuiMenuItemDef::caption => $move_fav_top_action['caption'],
            GuiMenuItemDef::action => $move_fav_top_action
        );
        $menu_items[] = array(
            GuiMenuItemDef::caption => $remove_fav_action['caption'],
            GuiMenuItemDef::action => $remove_fav_action
        );
        $popup_search_action = ActionFactory::show_popup_menu($menu_items);

        $skip_details = isset($plugin_cookies->skip_details) ? $plugin_cookies->skip_details : 'no';

        if ($skip_details === 'yes')
        {
            return array
                (
                GUI_EVENT_KEY_ENTER => UserInputHandlerRegistry::create_action($this, 'view_series'),
                GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
                GUI_EVENT_KEY_A_RED => $switch_view,
                GUI_EVENT_KEY_INFO => $view_info,
                GUI_EVENT_KEY_POPUP_MENU => $popup_search_action
            );
        }
        else
        {
            return array
                (
                GUI_EVENT_KEY_ENTER => $view_info,
                GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
                GUI_EVENT_KEY_A_RED => $switch_view,
                GUI_EVENT_KEY_POPUP_MENU => $popup_search_action
            );
        }
    }

    ///////////////////////////////////////////////////////////////////////
    public function get_video_action_map($media_url, &$plugin_cookies) 
    {
        //-//hd_print('--->>> 5--->>>' . $media_url);
		
			return array
            (
            GUI_EVENT_KEY_ENTER => self::$open_folder_action        
			);
		
    }

    ///////////////////////////////////////////////////////////////////////
    public function get_main_menu_action_map($media_url, &$plugin_cookies)
    {	//-//hd_print('--->>> 6--->>>' . $media_url);
		$ver = file_get_contents(dirname(__FILE__).'/VERSION');
		$ver = str_replace("﻿version=", "", $ver);
		$ver = str_replace('date=', "[", $ver);
		$ver = str_replace(" 00:00", "]", $ver);
		$ver = str_replace("\n", " ", $ver);
		
		$setup_view = UserInputHandlerRegistry::create_action(
                        $this, 'setup');
        $setup_view['caption'] = (is_interface_language_russian_or_similar() ? "Настройки" : 'Settings');
		
		$info_release = UserInputHandlerRegistry::create_action(
                        $this, 'info_release');
        $info_release['caption'] = (is_interface_language_russian_or_similar() ? "Изменения в v$ver" : 'Info Release');

        return array
            (
            GUI_EVENT_KEY_ENTER => self::$open_folder_action,
			GUI_EVENT_KEY_B_GREEN => $setup_view,
            GUI_EVENT_KEY_C_YELLOW => $info_release,
        );
    }

    ///////////////////////////////////////////////////////////////////////
    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        //-//hd_print('--> Entry handler: handle_user_input:');
        foreach ($user_input as $key => $value)
        //-//hd_print("$key => $value");
        update_interface_language();

        $do_search_action = UserInputHandlerRegistry::create_action($this, 'do_search');
        $do_new_search_action = UserInputHandlerRegistry::create_action($this, 'do_new_search');
        $perform_new_search_action = UserInputHandlerRegistry::create_action($this, 'perform_new_search');
        $do_add_search_action = UserInputHandlerRegistry::create_action($this, 'do_add_search');
        $do_mark_viewed = UserInputHandlerRegistry::create_action($this, 'mark_viewed');
        $add_fav_complite = UserInputHandlerRegistry::create_action($this, 'add_fav_complite');
        $del_fav_complite = UserInputHandlerRegistry::create_action($this, 'del_fav_complite');
        $just_play = UserInputHandlerRegistry::create_action($this, 'just_play');

        if (isset($user_input->control_id))
        {
            $control_id = $user_input->control_id;
            switch ($control_id)
            {
                case 'search':
                    $new_value = $user_input->{$control_id};
                    // last entered search
                    $plugin_cookies->search = $new_value;
                    $defs = $this->do_get_search_defs($plugin_cookies);
                    return  ActionFactory::reset_controls
                            (
                                $defs,
                                null,
                                1
                            );
//                    return null;
                case 'perform_new_search':
                    $new_value = $plugin_cookies->search;
                    HD::decode_user_data($user_input->parent_media_url, $media_str, $user_data);
					$search_ref = $user_data->search_ref;
					//-//hd_print('--->>> search_ref --->>>' . $search_ref);
                    $search_text = urlencode($new_value);
					if (preg_match("/p=1/i",$search_ref)){
					$search_text = $search_ref . '&search=' . $search_text;}
					else{
					$search_text = $search_ref . '?search=' . $search_text;}
                    $search_url = HD::encode_user_data(array('ser_feed' => $search_text));
					
                    return array
                    (
                        GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
                        GuiAction::data =>
                        array
                        (
                            'media_url' => $search_url
                        )
                    );
                case 'do_new_search':
                    $new_value = $plugin_cookies->search;
                    // search history
                    $search_items = isset($plugin_cookies->search_items) ? $plugin_cookies->search_items : '';
                    $plugin_cookies->search_items = ListUtil::add_item($search_items,$new_value);
                    return ActionFactory::invalidate_folders(array($user_input->parent_media_url), $perform_new_search_action);
                case 'do_search':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
                    if ($user_data->ser_feed != '')
                        break;

                    $defs = $this->do_get_search_defs($plugin_cookies);
                    return ActionFactory::show_dialog(
                            (is_interface_language_russian_or_similar() ? 'Поиск' : 'Search'),
                            $defs);
                    
                    break;
				case 'do_search_menu':
                    
					HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
					
                    $c = VideoCache::get($user_data->video_id);
                    HD::decode_user_data($c, $media_str, $user_data);
                    $title = '';
                    if (HD::has_attribute($user_data, 'season_title'))
                        $title = $user_data->season_title;
                    $plugin_cookies->search = $title;
                    $defs = $this->do_get_search_defs($plugin_cookies);
                    return ActionFactory::show_dialog(
                            (is_interface_language_russian_or_similar() ? 'Поиск' : 'Search'),
                            $defs);
                    
                    break;
                case 'skin':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->skin = $new_value;
                    shell_exec("cp -f /persistfs/plugins/cScVOD/skins/$new_value/icons/logo.png /persistfs/plugins/cScVOD/skins/logo.png");
                    break;
                case 'restart':
                    shell_exec('killall shell');
                    break;
				case 'user_youtube':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->user_youtube = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_control_defs($plugin_cookies)
                    );
					break;
				case 'info_release':
                   
					$post_action = null;
					
						$doc = HD::http_get_document('http://copy.com/8aMn0OK4hrZlxQk4/Dune/update/cScVOD/info.txt');
						$tmp = explode('>>>', $doc);
						$text = $tmp[0];
						$defs = array();
						$texts = explode("\n", $text);
						$texts = array_values($texts);
						foreach($texts as $text)
						{
						$text = str_remove_spec($text);
						ControlFactory::add_label($defs, "", $text);
						}
						ControlFactory::add_custom_close_dialog_and_apply_buffon($defs, 'setup', 'Ok', 150,  $post_action);

						return ActionFactory::show_dialog('Информация об изменениях.', $defs);
                    break;
				case 'setup':
                    return ActionFactory::open_folder('setup');
					break;
                case 'show_main_icon':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->main_icon = $new_value;
		            if ($new_value == 'yes')
		            {
			            $plugin_cookies->show_channels_v4 = 'no';
			            $plugin_cookies->show_search_v4 = 'no';
			            $plugin_cookies->show_favorites = 'no';
		            }
		            else
		            {
			            $plugin_cookies->show_channels_v4 = 'yes';
			            $plugin_cookies->show_search_v4 = 'yes';
			            $plugin_cookies->show_favorites = 'yes';
		            }
                    return ActionFactory::reset_controls(
                            $this->do_get_control_defs($plugin_cookies)
                    );
                    break;
                case 'video_quality':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->video_quality = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_control_defs($plugin_cookies)
                    );
                    break;
                case 'use_osk':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->use_osk = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_control_defs($plugin_cookies)
                    );
                    break;
                case 'skip_details':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->skip_details = $new_value;
                    return ActionFactory::reset_controls(
                            $this->do_get_control_defs($plugin_cookies)
                    );
                    break;
                case 'buf_time':
                    $new_value = $user_input->{$control_id};
                    $plugin_cookies->buf_time = $new_value;
                    return  ActionFactory::reset_controls
                            (
                                $this->do_get_control_defs($plugin_cookies)
                            );
                    break;
                case 'ex_view_info':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
                    
                    if (!HD::has_attribute($user_data, 'video_id'))
                        return null;
                        
                    return array
                        (
                             GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
                             GuiAction::data => array(
                                    'media_url' => HD::encode_user_data(
                                                        array(
                                                            'info' => '1',
                                                            'video_id' => $user_data->video_id,
                                                            'season_ref' => $user_data->video_id,
                                                            //'season_title' => $user_data->season_title,
                                                        )
                                                    )
                             )
                         );
                    break;
                    
                case 'add_fav_complite':
                    return ActionFactory::show_title_dialog(
                        (
                            is_interface_language_russian_or_similar() ? 'Видео добавлено в избранное' : "Video has been added to favorites"
                        )
                    );
                    break;
                 case 'del_fav_complite':
                    return ActionFactory::show_title_dialog(
                        (
                            is_interface_language_russian_or_similar() ? 'Видео было удалено из избранного' : "Video has been deleted from favorites"
                        )
                    );
                    break;
                   
                case 'add_favorite':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    if ($user_input->selected_media_url === 'right_button')
                        HD::decode_user_data($user_input->parent_media_url, $media_str, $user_data);
                    else
                        HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
		            
                    if (HD::has_attribute($user_data, 'video_id'))
                    	$video_id = $user_data->video_id;
                    else
                        return null;

                    $fav_items = isset($plugin_cookies->fav_items) ? $plugin_cookies->fav_items : '';

                    if (!ListUtil::is_in_list($fav_items, $video_id))
                    {
		                $plugin_cookies->fav_items = ListUtil::add_item($fav_items,$video_id);
                        $plugin_cookies->$video_id = "";
          
                        $items = $this->parse_season($video_id, $plugin_cookies);

                        foreach($items as $v)
                        {
                            HD::decode_user_data($v, $media_str, $udata);
                            
                            $s = $udata->episode_ref;

                            if (strlen($s) == 0)
                                continue;
                                
                            $i = $plugin_cookies->$video_id;
                            
                            $plugin_cookies->$video_id = ListUtil::add_item($i, $s);
                        }
                        
                        if ($user_input->selected_media_url === 'right_button')
                        {
                            return ActionFactory::invalidate_folders(array($user_input->parent_media_url), $add_fav_complite);
                        }
                        else
                        {
                            return ActionFactory::show_title_dialog('Видео добавлено в избранное');
                        }
                    }
                    else
                    {
                        return ActionFactory::show_title_dialog('Видео уже в избранном');
                    }
                    break;
                case 'remove_fav':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    if ($this->fav_build_in_progess)
                        return ActionFactory::show_title_dialog('Подождите окончания построения списка!');
                        
                    if ($user_input->selected_media_url === 'right_button')
                        HD::decode_user_data($user_input->parent_media_url, $media_str, $user_data);
                    else
                        HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);

                    if (HD::has_attribute($user_data, 'video_id'))
                        $video_id = $user_data->video_id;
                    else if (HD::has_attribute($user_data, 'fav'))
                        $video_id = $user_data->fav;
                    else
                        return null;

                    $fav_items = isset($plugin_cookies->fav_items) ? ListUtil::del_item($plugin_cookies->fav_items, $video_id) : '';
                    $plugin_cookies->fav_items = $fav_items;
                    if (isset($plugin_cookies->$video_id))
                    {
                        unset($plugin_cookies->$video_id);
                    }
                    //{ Viewed items
                    $s = $video_id . ".viewed";
                    if (isset($plugin_cookies->$s))
                    {
                        unset($plugin_cookies->$s);
                    }
                    //} Viewed items

                    $folder = FolderCache::get('main_menu:fav');
                    $folder->set_expired();
                    if ($user_input->selected_media_url === 'right_button')
                    {
                        return ActionFactory::invalidate_folders(array($user_input->parent_media_url), $del_fav_complite);
                    }
                    else
                    {
                        return ActionFactory::invalidate_folders(array($user_input->parent_media_url), null);
                    }
                    break;
                case 'move_fav_top':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    if ($this->fav_build_in_progess)
                        return ActionFactory::show_title_dialog('Подождите окончания построения списка!');
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
                    $video_id = $user_data->video_id;
                    $fav_items = isset($plugin_cookies->fav_items) ? $plugin_cookies->fav_items : '';
                    $plugin_cookies->fav_items = ListUtil::add_item($fav_items,$video_id);
                    return ActionFactory::invalidate_folders(array($user_input->parent_media_url), null);
                    break;
                case 'just_play':
                    return array(
                            GuiAction::handler_string_id => PLUGIN_VOD_PLAY_ACTION_ID,
                            GuiAction::data =>
                                array(
                                    'media_url' => $user_input->selected_media_url
                                )
                    );
                    break;
                case 'play_item':
                    if (!isset($user_input->selected_media_url))
                        return null;                 
                    return ActionFactory::invalidate_folders(array($user_input->selected_media_url, $user_input->parent_media_url), $just_play);
                    break;
		        case 'view_series':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);

		            $n_series = 0;
		            if (HD::has_attribute($user_data, 'n_series'))
					
			            $n_series = $user_data->n_series;
	
		            $ndx_series = -1;
		            if (HD::has_attribute($user_data, 'ndx_series'))
			            $ndx_series = $user_data->ndx_series;

		            if ($n_series < 2 || $ndx_series >= 0)
		            {
	                    return array(
        	                GuiAction::handler_string_id => PLUGIN_VOD_PLAY_ACTION_ID,
                	        GuiAction::data =>
                        	array(
	                            'media_url' => $user_input->selected_media_url
        	                )
			            );
		            }
		            else
		            {
	                    return array(
        	                GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
                	        GuiAction::data =>
                        	array(
	                            'media_url' => $user_input->selected_media_url
        	                )
			            );
                   }
		           break;
                case 'remove_search':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
                    $ser_feed = urldecode($user_data->ser_feed);
                    $search_items = isset($plugin_cookies->search_items) ? ListUtil::del_item($plugin_cookies->search_items, $ser_feed) : '';
                    $plugin_cookies->search_items = $search_items;
                    return ActionFactory::update_regular_folder(
                                    $this->get_regular_folder_items('main_menu:search', 0, $plugin_cookies), true);
                    break;
                case 'move_search_top':
                    if (!isset($user_input->selected_media_url))
                        return null;
                    HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
                    $ser_feed = urldecode($user_data->ser_feed);
                    $search_items = isset($plugin_cookies->search_items) ? $plugin_cookies->search_items : '';
                    $plugin_cookies->search_items = ListUtil::add_item($search_items,$ser_feed);
                    return ActionFactory::update_regular_folder(
                                    $this->get_regular_folder_items('main_menu:search', 0, $plugin_cookies), true);
                    break;
                case 'mark_viewed':
                    //'url_series'
                    if (isset($user_input->selected_media_url))
                    {
                        HD::decode_user_data($user_input->selected_media_url, $media_str, $user_data);
                        if (HD::has_attribute($user_data, 'video_id'))
                            $video_id = $user_data->video_id;
                        else if (HD::has_attribute($user_data, 'season_ref'))
                            $video_id = $user_data->season_ref;
                        
                            if (isset($plugin_cookies->$video_id))
                            {
                                if (HD::has_attribute($user_data, 'url_series'))
                                    $url = $user_data->url_series;
                                else if (HD::has_attribute($user_data, 'episode_ref'))
                                    $url = $user_data->episode_ref;
                                if (!ListUtil::is_in_list($plugin_cookies->$video_id, $url))
                                {
                                    $viewed = $plugin_cookies->$video_id;
                                    $plugin_cookies->$video_id = ListUtil::add_item($viewed,$url);
                                }
                            }
							//{ Viewed items
                            
                            $s = $video_id . ".viewed";
                            if (!isset($plugin_cookies->$s))
                                $plugin_cookies->$s = "";
                            if (HD::has_attribute($user_data, 'url_series'))
                                $url = $user_data->url_series;
                            else if (HD::has_attribute($user_data, 'episode_ref'))
                                $url = $user_data->episode_ref;
                            if (!ListUtil::is_in_list($plugin_cookies->$s, $url))
                            {
                                $i = $plugin_cookies->$s;       
                                $plugin_cookies->$s = ListUtil::add_item($i, $url);
                            }
							else
							{
								$i = $plugin_cookies->$s; 
								$plugin_cookies->$s = ListUtil::del_item($i, $url);
							}
                            //} Viewed items

                        
                    }
                    return ActionFactory::update_regular_folder(
                                    $this->get_regular_folder_items($user_input->parent_media_url, 0, $plugin_cookies), true);
                    break;
            }
        }
        return array
            (
            GuiAction::handler_string_id => PLUGIN_OPEN_FOLDER_ACTION_ID,
            GuiAction::data => array(
                'media_url' => $user_input->selected_media_url
                    )
             );
    }
    ///////////////////////////////////////////////////////////////////////


    protected function add_text_field(&$defs, $name, $title, $initial_value, $numeric, $password, $has_osk, $always_active, $width, $need_confirm = false, $need_apply = false) {
        ControlFactory::add_text_field($defs, $this, null,
                        $name, $title, $initial_value,
                        $numeric, $password, $has_osk, $always_active, $width,
                        $need_confirm, $need_apply);
    }

    ///////////////////////////////////////////////////////////////////////
    protected function add_combobox(&$defs, $name, $title, $initial_value, $value_caption_pairs, $width, $need_confirm = false, $need_apply = false)
    {
        ControlFactory::add_combobox($defs, $this, null,
                        $name, $title, $initial_value, $value_caption_pairs, $width,
                        $need_confirm, $need_apply);
    }

    protected function add_button(&$defs,$name, $title, $caption, $width)
    {
        ControlFactory::add_button($defs, $this, null,
            $name, $title, $caption, $width);
    }

    ///////////////////////////////////////////////////////////////////////
    public function do_get_control_defs(&$plugin_cookies)
    {
        $defs = array();

        $main_icon = isset($plugin_cookies->main_icon) ? $plugin_cookies->main_icon : 'yes';
        $use_osk = isset($plugin_cookies->use_osk) ? $plugin_cookies->use_osk : 'yes';
        $skip_details = isset($plugin_cookies->skip_details) ? $plugin_cookies->skip_details : 'no';
        $video_quality = isset($plugin_cookies->video_quality) ? $plugin_cookies->video_quality : 1;
        $buf_time = isset($plugin_cookies->buf_time) ? $plugin_cookies->buf_time : 0;
		$user_youtube = isset($plugin_cookies->user_youtube) ? $plugin_cookies->user_youtube : '';
        $vquality = $this->get_quality_array(true);
        
		$ver = file_get_contents(dirname(__FILE__).'/VERSION');
		$ver = str_replace("﻿version=", "", $ver);
		$ver = str_replace('date=', "[", $ver);
		$ver = str_replace(" 00:00", "]", $ver);
		$ver = str_replace("\n", " ", $ver);
		
        $show_ops = array();
        if (is_interface_language_russian_or_similar ()) {
            $show_ops['yes'] = 'Да';
            $show_ops['no'] = 'Нет';
        } else {
            $show_ops['yes'] = 'Yes';
            $show_ops['no'] = 'No';
        }
        
        $this->add_button
        (
            $defs,
            'info_release',
            "Версия: $ver",
            'Информация об изменениях',
            0
        );
        $this->add_text_field($defs,
                    'user_youtube',
                    'Логин пользователя Youtube:',
                    $user_youtube, 0, 0, 0, 1, 400, 0, true
            );

        $this->add_combobox($defs,
                'show_main_icon',
                'Показывать иконку в главном меню:',
                $main_icon, $show_ops, 0, true
        );

        $this->add_combobox($defs,
                'video_quality',
                'Качество видео:',
                $video_quality, $vquality, 0, true
        );
        
	    $this->add_combobox($defs,
        	       'use_osk',
                'Использовать экранную клавиатуру в поиске:',
                $use_osk, $show_ops, 0, true
            );


        $this->add_combobox($defs,
                'skip_details',
                'Пропускать описание фильма:',
                $skip_details, $show_ops, 0, true
        );
 
        $show_buf_time_ops = array();
        
        $show_buf_time_ops[0] = 'По умолчанию';
        $show_buf_time_ops[500] = '0.5 с';
        $show_buf_time_ops[1000] = '1 с';
        $show_buf_time_ops[2000] = '2 с';
        $show_buf_time_ops[3000] = '3 с';
        $show_buf_time_ops[5000] = '5 с';
        $show_buf_time_ops[10000] = '10 с';

        $this->add_combobox
        (
            $defs,
            'buf_time',
            'Время буферизации:',
            $buf_time, $show_buf_time_ops, 0, true
        );
        $skin_ops = array();

        $skin_ops['default'] = 'По умолчанию';
        $skin_ops['juniperus'] = 'От Juniperus (Пиратская тема)';

        $skin = isset($plugin_cookies->skin) ? $plugin_cookies->skin : 'default';
        $this->add_combobox
        (
            $defs,
            'skin',
            'Оформление (требуется перезагрузка):',
            $skin, $skin_ops, 0, true
        );

        $this->add_button
        (
            $defs,
            'restart',
            '',
            'Перезагрузить плеер',
            0
        );

       return $defs;
    }

    ///////////////////////////////////////////////////////////////////////
    public function do_get_search_defs(&$plugin_cookies)
    {
        $defs = array();
        $search_text = isset($plugin_cookies->search) ? $plugin_cookies->search : '';
        $use_osk = isset($plugin_cookies->use_osk) ? $plugin_cookies->use_osk : 'yes';
        $this->add_text_field($defs,
                'search', "",
                $search_text, 0, 0,
                ($use_osk === 'yes' ? 1 : 0),
                true, 1300, 0, true
        );
        if ($use_osk === 'yes')
            ControlFactory::add_vgap($defs, 500);
        else
            ControlFactory::add_vgap($defs, 50);

        $do_new_search_action = UserInputHandlerRegistry::create_action($this, 'do_new_search');

        ControlFactory::add_custom_close_dialog_and_apply_buffon($defs,'apply_subscription',
                (is_interface_language_russian_or_similar() ? 'Искать' : 'Search'),
                300, $do_new_search_action);
        ControlFactory::add_close_dialog_button($defs,
                'Отмена', 300);

        return $defs;
    }

    ///////////////////////////////////////////////////////////////////////
    public function get_handler_id()
    {
        return self::ID;
    }

    ///////////////////////////////////////////////////////////////////////
    public function get_next_folder_view($media_url, &$plugin_cookies)
    {
        //-//hd_print('get_next_folder_view');
        $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
        if ($view_type == 3)
            $view_type = 1;
        else
            ++$view_type;
            
        $plugin_cookies->view_type = $view_type;
        
        return $this->get_folder_view($media_url, $plugin_cookies); 
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_folder_view($media_url, &$plugin_cookies)
    {
	    //-//hd_print('--->>> get_folder_view --->>>' . $media_url);
		HD::decode_user_data($media_url, $media_str, $user_data);
	    if ($media_url === 'main_menu')
	    {
            return  array
                    (
                        PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
                        PluginFolderView::multiple_views_supported  => false,
                        PluginFolderView::data                      => array
                        (
                            PluginRegularFolderView::async_icon_loading => false,
                            PluginRegularFolderView::initial_range =>
                                $this->get_main_menu_items($media_url, $plugin_cookies),
                            PluginRegularFolderView::view_params => self::get_main_menu_folder_view($plugin_cookies),
                            PluginRegularFolderView::base_view_item_params => self::get_main_vip($plugin_cookies),
                            PluginRegularFolderView::not_loaded_view_item_params => array(),
                            PluginRegularFolderView::actions => $this->get_main_menu_action_map($media_url, $plugin_cookies)
                        )
                    );
	    }
        else if 
	    (
		    $media_url === 'main_menu:videos' ||
		    $media_url === 'main_menu:search'
	    )
        {
		    switch ($media_url) 
		    {
                    case 'main_menu:search':
                        $actions = $this->get_search_action_map($media_url, $plugin_cookies);
                        break;
                    case 'main_menu:videos':
                        $actions = $this->get_video_action_map($media_url, $plugin_cookies);
                        break;
		    }

            return  array
                    (
                        PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
                        PluginFolderView::multiple_views_supported  => false,
                        PluginFolderView::data                      => array
                        (
                            PluginRegularFolderView::async_icon_loading => false,
                            PluginRegularFolderView::initial_range =>
                                $this->get_main_menu_items($media_url, $plugin_cookies),
                            PluginRegularFolderView::view_params => array
                            (
                                ViewParams::num_cols => 1,
                                ViewParams::num_rows => 12
                            ),
                            PluginRegularFolderView::base_view_item_params => self::$catalog_vip,
                            PluginRegularFolderView::not_loaded_view_item_params => array(),
                            PluginRegularFolderView::actions => $actions
                        )
                    );
	    }
	    elseif ($media_url === 'setup')
	    {
                $defs = $this->do_get_control_defs($plugin_cookies);
                $folder_view = array
                    (
                	    PluginControlsFolderView::defs => $defs,
                	    PluginControlsFolderView::initial_sel_ndx => -1,
            	    );
                return array
                    (
                	    PluginFolderView::multiple_views_supported => false,
	                    PluginFolderView::archive => null,
        	            PluginFolderView::view_kind => PLUGIN_FOLDER_VIEW_CONTROLS,
                	    PluginFolderView::data => $folder_view,
            	    );
            }
		elseif
         (
            HD::has_attribute($user_data, 'search_ref')
         )
		 {
		 $actions = $this->get_search_action_map($media_url, $plugin_cookies);
		 return  array
                    (
                        PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
                        PluginFolderView::multiple_views_supported  => false,
                        PluginFolderView::data                      => array
                        (
                            PluginRegularFolderView::async_icon_loading => false,
                            PluginRegularFolderView::initial_range =>
                                $this->get_main_menu_items($media_url, $plugin_cookies),
                            PluginRegularFolderView::view_params => array
                            (
                                ViewParams::num_cols => 1,
                                ViewParams::num_rows => 12
                            ),
                            PluginRegularFolderView::base_view_item_params => self::$catalog_vip,
                            PluginRegularFolderView::not_loaded_view_item_params => array(),
                            PluginRegularFolderView::actions => $actions
                        )
                    );
		 }
         $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;

		
         if
         (
            HD::has_attribute($user_data, 'category_ref')||
			HD::has_attribute($user_data, 'site_ref')
         )
         {
             if (HD::has_attribute($user_data, 'ser_feed'))
             {
                $actions = $this->get_item_action_map($media_url, $plugin_cookies);
                $not_loaded_item = self::$catalog_vip;
             }
             else if ($media_url === 'main_menu:fav')
             {
                $actions = $this->get_fav_action_map($media_url, $plugin_cookies);
                $not_loaded_item = self::$catalog_vip;
             }
             else
             {
                $actions = $this->get_video_action_map($media_url, $plugin_cookies);
                $not_loaded_item = array();
             }

            return  array
                    (
                        PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
                        PluginFolderView::multiple_views_supported  => false,
                        PluginFolderView::data                      => array
                        (
                            PluginRegularFolderView::async_icon_loading => false,
                            PluginRegularFolderView::initial_range =>
                                $this->get_regular_folder_items($media_url, 0, $plugin_cookies),
                            PluginRegularFolderView::view_params => array
                            (
                                ViewParams::num_cols => 1,
                                ViewParams::num_rows => 12
                            ),
                            PluginRegularFolderView::base_view_item_params => self::$catalog_vip,
                            PluginRegularFolderView::not_loaded_view_item_params => $not_loaded_item,
                            PluginRegularFolderView::actions => $actions
                        )
                    );
         }
         else if 
         (
            $media_url === 'main_menu:fav' ||
            HD::has_attribute($user_data, 'subcategory_ref') ||
            HD::has_attribute($user_data, 'ser_feed')
         )
         {
//            $actions = $this->get_video_action_map($media_url, $plugin_cookies);
             if ($media_url === 'main_menu:fav')
             {
                $actions = $this->get_fav_action_map($media_url, $plugin_cookies);
             }
             else
             {
                $actions = $this->get_item_action_map($media_url, $plugin_cookies);
             }

            return array(
                PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
                PluginFolderView::multiple_views_supported  => true,
                PluginFolderView::data                      => array(
                    PluginRegularFolderView::async_icon_loading => true,
                    PluginRegularFolderView::initial_range =>
                        $this->get_loading_items($plugin_cookies),

                    PluginRegularFolderView::view_params => array(
                        ViewParams::num_cols => ($view_type == 1 ? 5 : 1),
                        ViewParams::num_rows => ($view_type == 1 ? 2 : ($view_type == 2 ? 3 : ($view_type == 4 ? 1 : 10))),
                        ViewParams::paint_details => ($view_type == 4 ? false : true),
                        ViewParams::zoom_detailed_icon => true,
                        ViewParams::paint_item_info_in_details => true,
                        ViewParams::item_detailed_info_font_size => FONT_SIZE_NORMAL,// FONT_SIZE_SMALL,
                        
                        ViewParams::paint_sandwich => (($view_type == 1 || $view_type == 4) ? true : false),
                        ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                        ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                        ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                        ViewParams::sandwich_width => 190,
                        ViewParams::sandwich_height => 290,
                        ViewParams::sandwich_icon_upscale_enabled => true,
                        ViewParams::sandwich_icon_keep_aspect_ratio => true,
                    ),

                    PluginRegularFolderView::base_view_item_params => array(
                        ViewItemParams::item_paint_icon => true,
                        ViewItemParams::icon_sel_scale_factor =>1.2,
                        ViewItemParams::icon_path => 'gui_skin://small_icons/movie.aai',
                        ViewItemParams::item_layout => HALIGN_LEFT,
                        ViewItemParams::icon_valign => VALIGN_CENTER,
                        ViewItemParams::icon_dx => 10,
                        ViewItemParams::icon_dy => -5,
                        ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : ($view_type == 4 ? 250 : 50))),
                        ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : ($view_type == 4 ? 350 : 50))),
                        ViewItemParams::icon_sel_margin_top => 0,
                        ViewItemParams::item_paint_caption => ($view_type == 1 ? false : true),
                        ViewItemParams::item_caption_width => ($view_type == 2 ? 950 : 1100)
                    ),

                    PluginRegularFolderView::not_loaded_view_item_params => array(
                        ViewItemParams::item_paint_icon => true,
                        ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                        ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                        ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai')
                    ),

                    PluginRegularFolderView::actions => $actions
                )
            );
         }
         else if
         (
            HD::has_attribute($user_data, 'season_ref') &&
            !HD::has_attribute($user_data, 'info')
         )
{
                if (isset($plugin_cookies->fav_items))
                $isfav = ListUtil::is_in_list($plugin_cookies->fav_items, $user_data->season_ref);

            
            if ($isfav)
            {
                    $add_action = UserInputHandlerRegistry::create_action(
                                $this, 'mark_viewed');
                    $add_action['caption'] = (
                                is_interface_language_russian_or_similar() ? 'Просмотрено/Не просмотрено' : "Mark as viewed/not viewed"
                            );
					
                $actions = array
                    (
                        GUI_EVENT_KEY_ENTER => $this->get_play_action($media_url, $plugin_cookies),
                        GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
                        GUI_EVENT_KEY_B_GREEN => $add_action
                    );
            }
            else
            {
					    $add_action = UserInputHandlerRegistry::create_action(
                                $this, 'mark_viewed');
						$add_action['caption'] = (
                                is_interface_language_russian_or_similar() ? 'Просмотрено/Не просмотрено' : "Mark as viewed/not viewed"
                            );
						
				$actions = array
                (
						GUI_EVENT_KEY_ENTER => $this->get_play_action($media_url, $plugin_cookies),
                        GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
						GUI_EVENT_KEY_B_GREEN => $add_action
                );
            }

            return array
            (
                PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
                PluginFolderView::multiple_views_supported  => false,
                PluginFolderView::data                      => array(
                    PluginRegularFolderView::async_icon_loading => true,
                    PluginRegularFolderView::initial_range =>
                        $this->get_regular_folder_items($media_url, 0, $plugin_cookies),

                    PluginRegularFolderView::view_params => array(
                        ViewParams::num_cols => 1,
                        ViewParams::num_rows => 10,
                        ViewParams::paint_details => false,
                    ),

                    PluginRegularFolderView::base_view_item_params => array(
                        ViewItemParams::item_paint_icon => true,
                        ViewItemParams::icon_path => 'gui_skin://small_icons/movie.aai',
                    ViewItemParams::item_layout => HALIGN_LEFT,
                       ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::icon_dx => 20,
                    ViewItemParams::icon_dy => -5,
                    ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
                    ViewItemParams::item_caption_width => 1550
                    ),

                    PluginRegularFolderView::not_loaded_view_item_params => array(),

                    PluginRegularFolderView::actions => $actions
                )
            );
         }
         
         if (HD::has_attribute($user_data, 'info'))
         {
            $c = $this->parse_season_info($user_data->video_id);
            HD::decode_user_data($c, $media_str, $user_data);

            $movie = array(
                PluginMovie::name => $user_data->title,
                PluginMovie::name_original => $user_data->name_original,
                PluginMovie::description => $user_data->description,
                PluginMovie::poster_url => $user_data->video_image_url,
                PluginMovie::length_min => $user_data->duration_min,
                PluginMovie::year => $user_data->year,
                PluginMovie::directors_str => $user_data->director,
                PluginMovie::scenarios_str => '',
                PluginMovie::actors_str => $user_data->actors,
                PluginMovie::genres_str => $user_data->genre,
                PluginMovie::rate_imdb => $user_data->imdb_rating,
                PluginMovie::rate_kinopoisk => $user_data->kp_rating,
                PluginMovie::rate_mpaa => '',
                PluginMovie::country => $user_data->country,
                PluginMovie::budget => ''
                );
                

            $from_fav = false;
            $fav_items = isset($plugin_cookies->fav_items) ? $plugin_cookies->fav_items : '';
            if (ListUtil::is_in_list($fav_items, $user_data->video_id))
                $from_fav = true;
                
            if ($from_fav)
            {
                $right_button_action = UserInputHandlerRegistry::create_action(
                            $this, 'remove_fav');
                $right_button_action['caption'] = (
                            is_interface_language_russian_or_similar() ? "Удалить из Избранного" : 'Remove from Favorites'
                        );
            }
            else
            {
                $right_button_action = UserInputHandlerRegistry::create_action(
                            $this, 'add_favorite');
                $right_button_action['caption'] = (
                            is_interface_language_russian_or_similar() ? 'Добавить в Избранное' : "Add Favorite"
                        );
            }


            $right_button_caption = $right_button_action['caption'];
            
            $movie_folder_view = array
            (
                PluginMovieFolderView::movie => $movie,
                PluginMovieFolderView::has_right_button => true,
                PluginMovieFolderView::right_button_caption => $right_button_caption,
                PluginMovieFolderView::right_button_action => $right_button_action,
                PluginMovieFolderView::has_multiple_series => ($user_data->n_series > 1),
                PluginMovieFolderView::series_media_url => HD::encode_user_data(
                                                                array(
                                                                    'season_ref' => $user_data->video_id,
                                                                    'season_title' => $user_data->title,
                                                                    'n_series' => $user_data->n_series,
                                                                )
                                                           )
            );

             return array
                (
                    PluginFolderView::multiple_views_supported => false,
                    PluginFolderView::archive => null,
                    PluginFolderView::view_kind => PLUGIN_FOLDER_VIEW_MOVIE,
                    PluginFolderView::data => $movie_folder_view
                );
         }
         else if (HD::has_attribute($user_data, 'video_id'))
         {
	        $isfav = ListUtil::is_in_list($plugin_cookies->fav_items, $user_data->video_id);
            
            if ($isfav)
            {
                    $add_action = UserInputHandlerRegistry::create_action(
                                $this, 'mark_viewed');
                    $add_action['caption'] = (
                                is_interface_language_russian_or_similar() ? 'Просмотрено/Не просмотрено' : "Mark as viewed/not viewed"
                            );		
                $actions = array
                    (
                        GUI_EVENT_KEY_ENTER => $this->get_play_action($media_url, $plugin_cookies),
                        GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
                        GUI_EVENT_KEY_B_GREEN => $add_action,
                    );
            }
            else
            {
					    $add_action = UserInputHandlerRegistry::create_action(
                                $this, 'mark_viewed');
						$add_action['caption'] = (
                                is_interface_language_russian_or_similar() ? 'Просмотрено/Не просмотрено' : "Mark as viewed/not viewed"
                            );

				$actions = array
                (
						GUI_EVENT_KEY_ENTER => $this->get_play_action($media_url, $plugin_cookies),
                        GUI_EVENT_KEY_PLAY => $this->get_play_action($media_url, $plugin_cookies),
						GUI_EVENT_KEY_B_GREEN => $add_action
                );
            }
            
            return array(
                PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
                PluginFolderView::multiple_views_supported  => false,
                PluginFolderView::data                      => array(
                    PluginRegularFolderView::async_icon_loading => true,
                    PluginRegularFolderView::initial_range =>
                        $this->get_regular_folder_items($media_url, 0, $plugin_cookies),

                    PluginRegularFolderView::view_params => array(
                        ViewParams::num_cols => 1,
                        ViewParams::num_rows => 10,
                        ViewParams::paint_details => false,
                    ),

                    PluginRegularFolderView::base_view_item_params => array(
                        ViewItemParams::item_paint_icon => true,
                        ViewItemParams::icon_path => 'gui_skin://small_icons/movie.aai',
	                ViewItemParams::item_layout => HALIGN_LEFT,
       		        ViewItemParams::icon_valign => VALIGN_CENTER,
        	        ViewItemParams::icon_dx => 20,
        	        ViewItemParams::icon_dy => -5,
        	        ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
        	        ViewItemParams::item_caption_width => 1550
                    ),

                    PluginRegularFolderView::not_loaded_view_item_params => array(),

                    PluginRegularFolderView::actions => $actions
                )
            );
         }

	    if ($media_url === 'main_menu:fav')
            $actions = $this->get_fav_action_map($media_url, $plugin_cookies);
	    else
	        $actions = $this->get_item_action_map($media_url, $plugin_cookies);

        return array(
            PluginFolderView::view_kind                 => PLUGIN_FOLDER_VIEW_REGULAR,
            PluginFolderView::multiple_views_supported  => true,
            PluginFolderView::data                      => array(
                PluginRegularFolderView::async_icon_loading => true,
                PluginRegularFolderView::initial_range =>
                    $this->get_loading_items($plugin_cookies),

                PluginRegularFolderView::view_params => array(
                    ViewParams::num_cols => ($view_type == 1 ? 5 : 1),
                    ViewParams::num_rows => ($view_type == 1 ? 2 : ($view_type == 2 ? 3 : 10)),
                    ViewParams::paint_details => true,
                    ViewParams::zoom_detailed_icon => true,
                    ViewParams::paint_item_info_in_details => true,

                    ViewParams::item_detailed_info_font_size => FONT_SIZE_NORMAL,
                    
                    ViewParams::paint_sandwich => ($view_type == 1 ? true : false),
                    ViewParams::sandwich_base => 'gui_skin://special_icons/sandwich_base.aai',
                    ViewParams::sandwich_mask => 'cut_icon://{name=sandwich_mask}',
                    ViewParams::sandwich_cover => 'cut_icon://{name=sandwich_cover}',
                    ViewParams::sandwich_width => 190,
                    ViewParams::sandwich_height => 290,
                    ViewParams::sandwich_icon_upscale_enabled => true,
                    ViewParams::sandwich_icon_keep_aspect_ratio => true,
                ),

                PluginRegularFolderView::base_view_item_params => array(
                    ViewItemParams::item_paint_icon => true,
                    ViewItemParams::icon_sel_scale_factor =>1.2,
                    ViewItemParams::icon_path => 'gui_skin://small_icons/movie.aai',
                    ViewItemParams::item_layout => HALIGN_LEFT,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::icon_dx => 10,
                    ViewItemParams::icon_dy => -5,
                    ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                    ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                    ViewItemParams::icon_sel_margin_top => 0,
                    ViewItemParams::item_paint_caption => ($view_type == 1 ? false : true),
                    ViewItemParams::item_caption_width => ($view_type == 2 ? 950 : 1100)
                ),

                PluginRegularFolderView::not_loaded_view_item_params => array(
                    ViewItemParams::item_paint_icon => true,
                    ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                    ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                    ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai')
                ),

                PluginRegularFolderView::actions => $actions
            )
        );

        throw new Exception('Unknown media url');
    }
	private function parse_category($category, &$plugin_cookies)
    {	//hd_print('--->>> parse_category --->>>');
        $user_youtube = isset($plugin_cookies->user_youtube) ? $plugin_cookies->user_youtube : '';
		$url = $category->category_ref;
		$doc = HD::http_get_document($url);
		$doc = str_replace('<![CDATA[','',$doc);
		$doc = str_replace(']]>','',$doc);
		$videos = explode('<title>', $doc);
		unset($videos[0]);
		$videos = array_values($videos);
        $items = array();
        foreach($videos as $video)
        {
		$tmp = explode('<playlist_url>', $video);
		$site_ref = strstr($tmp[1], '</playlist_url>', true);
		$s_image = 'plugin_file://skins/logo.png';
		$tmp = explode('<img src="', $video);
			if (count($tmp) > 1)
		$s_image = str_remove_spec(strstr($tmp[1], '"', true));
		$s_desc = 'cScVOD';
			$tmp = explode('<description>', $video);
			if (count($tmp) > 1)
			$s_desc = strip_tags(str_remove_spec(strstr($tmp[1], '<description>', true)));
        $site_title =  strstr($video, '</title>', true);
		if (preg_match("/^Поиск/i",$site_title)){
			array_push
            (
                $items,
                HD::encode_user_data
                (
					array
                    (
						'search_ref' => $site_ref,
						'site_title' => $site_title
                    )
                )
            );
			}
			elseif (preg_match("/vod\/iptv\/axenttv.php/i",$site_ref)){
			array_push
            (
                $items,
                HD::encode_user_data
                (
					array
                    (
                        'site_title' => $site_title,
                        'site_ref' => $site_ref
                    )
                )
            );
            ++$this->m_items_count;
			}
			elseif (preg_match("/vod\/iptv/i",$site_ref)||preg_match("/iptv.lamp.ufa/i",$site_ref) || preg_match("/iptv\/free/i",$site_ref)){
			$site_ref =  $site_ref . '&sinf&' . $site_title . '&sinf&' . $s_image . '&sinf&' . $s_desc;
			array_push
            (
                $items,
                HD::encode_user_data
                (
					array
                    (
						'season_ref' => $site_ref,
						'site_title' => $site_title
                    )
                )
            );
			}
			elseif (preg_match("/^Последние добавленые пользователем/i",$site_title) || preg_match("/^Плейлисты пользователя/i",$site_title)){
			if (!$user_youtube==''){
			$site_ref = $site_ref. '?search=' .$user_youtube;
            if (preg_match("/^Последние добавленые пользователем/i",$site_title)){
			array_push
            (
                $items,
                HD::encode_user_data
                (
					array
                    (
                        'site_title' => $site_title,
                        'subcategory_ref' => $site_ref
                    )
                )
            );
            ++$this->m_items_count;
			} else {
			array_push
            (
                $items,
                HD::encode_user_data
                (
					array
                    (
                        'site_title' => $site_title,
                        'site_ref' => $site_ref
                    )
                )
            );
            ++$this->m_items_count;
			}
        }
			else{
			array_push
            (
                $items,
                HD::encode_user_data
                (
					array
                    (
						'search_ref' => $site_ref,
						'site_title' => $site_title
                    )
                )
            );
			}}
			else{
            array_push
            (
                $items,
                HD::encode_user_data
                (
					array
                    (
                        'site_title' => $site_title,
                        'site_ref' => $site_ref
                    )
                )
            );
            ++$this->m_items_count;
        }}
        return $items;
    }
	private function parse_site($site)
    {	//-//hd_print('--->>> parse_site --->>>');
        $url = $site->site_ref;
		$doc = HD::http_get_document($url);
		$doc = str_replace("<title>Назад", "", $doc);
		$doc = str_replace('<![CDATA[','',$doc);
		$doc = str_replace(']]>','',$doc);
		$videos = explode('<title>', $doc);
		unset($videos[0]);
		$videos = array_values($videos);

        $items = array();
    
        foreach($videos as $video)
        {
		
		$tmp = explode('<playlist_url>', $video);
		$subcategory_ref = strstr($tmp[1], '</playlist_url>', true);
        $subcategory_title =  strstr($video, '</title>', true);
		if (preg_match("/^Поиск/i",$subcategory_title)){
			array_push
            (
                $items,
                HD::encode_user_data
                (
					array
                    (
						'search_ref' => $subcategory_ref,
						'subcategory_title' => $subcategory_title
                    )
                )
            );
			}
			else{
            array_push
            (
                $items,
                HD::encode_user_data
                (
					array
                    (
                        'subcategory_title' => $subcategory_title,
                        'subcategory_ref' => $subcategory_ref
                    )
                )
            );
            ++$this->m_items_count;
        }}
        return $items;
    }
    ///////////////////////////////////////////////////////////////////////
    private function parse_season_info($season_ref)
    {
        //-//hd_print("--> parse season info: started...");
		$tmp = explode('&sinf&', $season_ref);
        $url = $tmp[0];
		$s_titl = $tmp[1];
		$s_img = $tmp[2];
		$s_desc = $tmp[3];
        $api = HOST_API_URL;
        
        $items = array();

		if (preg_match("/kinoprosmotr.net/i",$season_ref)){
		$doc = HD::http_get_document($url);
		$doc = str_replace('<![CDATA[','',$doc);
		$doc = str_replace(']]>','',$doc);
		$doc = iconv('windows-1251', 'UTF-8', $doc);
		// title
        $video_title = $s_titl;
		$tmp = explode('<span>Название:', $doc);
		if (count($tmp) > 1)
        {
        $video_title = str_remove_spec(strstr($tmp[1], '</li>', true));
		$video_title = strip_tags($video_title);
		}
        // image
        $video_image = $s_img;
		$tmp = explode('<p class="full_movie_poster_img"><img src="', $doc);
		if (count($tmp) > 1)
        {
        $video_image = 'http://kinoprosmotr.net' . str_remove_spec(strstr($tmp[1], '"', true));
		}

        // description
        $description = $s_desc;
        $tmp = explode('<dt itemprop="description">', $doc);
        if (count($tmp) > 1)
        {
            $tmp = strstr($tmp[1], '</dt>', true);
            $description = str_remove_spec($tmp);
			$description = strip_tags($description);
       }

         // // ////-//hd_print("--->>> $description");
        // year
        $year = '';
        $tmp = explode('<span>Год выхода:', $doc);
        if (count($tmp) > 1)
            $year = strstr($tmp[1], '</li>', true);
        $year = strip_tags($year);
        //////-//hd_print("year--->>> $year");

        // country
        $country = '';
        $tmp = explode("<span>Страна:", $doc);
        if (count($tmp) > 1)
            $country = str_remove_spec(strstr($tmp[1], '</li>', true));
		$country = strip_tags($country);
         // // ////-//hd_print("--->>> $country");

        // genre
        $genre = '';
		$tmp = explode('<span>Жанр:', $doc);
        $genre = str_remove_spec(strstr($tmp[1], '</li>', true));
		$genre = strip_tags($genre);
		 // // ////-//hd_print("--->>> $genre");
        
        // duration
       $tmp = explode('Время:</span> ', $doc);
       $duration_min = 0;
       $duration = '';
       if (count($tmp) > 1)
		{
			$duration_min = strstr($tmp[1], ' мин', true);
			$duration = "$duration_min мин.";
		}
        
		 // // ////-//hd_print("--->>> $duration");

        // director
        $director = '';
        $tmp = explode('<span>Режиссер:', $doc);
        if (count($tmp) > 1)
        $director = str_remove_spec(strstr($tmp[1], '</li>', true));
		$director = strip_tags($director);
         // // ////-//hd_print("--->>> $director");
            
        // actors
        $actors = '';
        $tmp = explode('<span>В ролях:', $doc);
        if (count($tmp) > 1)
            $actors = str_remove_spec(strstr($tmp[1], '</li>', true));
        $actors = strip_tags($actors);
        // ////-//hd_print("--->>> $actors");
		
		$kp_rating = '';
		$imdb_rating = '';
		$tmp = explode('IMDb</noindex>: ', $doc);
		if (count($tmp) > 1)
			$imdb_rating = str_remove_spec(strstr($tmp[1], "(", true));
		$imdb_rating = strip_tags($imdb_rating);
		//////-//hd_print("kp_rating--->>> $kp_rating");
		//////-//hd_print("imdb_rating--->>> $imdb_rating");
		
        // number of series
        $n_series = 2;
		
		//name_original 
		$name_original = '';
        $tmp = explode("<span>Оригинальное название:", $doc);
        if (count($tmp) > 1)
            $name_original = str_remove_spec(strstr($tmp[1], '</li>', true));
			$name_original = strip_tags($name_original);
        //////-//hd_print("name_original--->>> $name_original");
		}
		elseif (preg_match("/http:\/\/vk.com\/video_ext.php/i",$season_ref)){
        $video_title = $s_titl;
        $video_image = $s_img;
        $description = $s_desc;
        $year = '';
        $country = '';
        $genre = '';
		$duration_min = 0;
		$duration = '';
        $director = '';
        $actors = '';
		$kp_rating = '';
		$imdb_rating = '';
        $n_series = 1;
		$name_original = '';
		}
		elseif (preg_match("/youtube.com\/watch/i",$season_ref)){
        $doc = HD::http_get_document($url);
		$video_title = $s_titl;
		$tmp = explode('<title>', $doc);
		if (count($tmp) > 1)
        {
        $video_title = str_remove_spec(strstr($tmp[1], '</title>', true));
		}
        $video_image = $s_img;
		$tmp = explode('<meta property="og:image" content="', $doc);
		if (count($tmp) > 1)
        {
        $video_image = str_remove_spec(strstr($tmp[1], '"', true));
		}
        $description = $s_desc;
        $tmp = explode('<meta name="twitter:description" content="', $doc);
        if (count($tmp) > 1)
        {
            $tmp = strstr($tmp[1], '"', true);
            $description = str_remove_spec($tmp);
			$description = strip_tags($description);
       }
        $year = '';
        $country = '';
        $genre = '';
		$duration_min = 0;
		$duration = '';
        $director = '';
        $actors = '';
		$kp_rating = '';
		$imdb_rating = '';
        $n_series = 1;
		$name_original = '';
		}
		else
		{
        $doc = HD::http_get_document($url);
		
		$doc = str_replace('<![CDATA[','',$doc);
		$doc = str_replace(']]>','',$doc);
        $video_title = $s_titl;
		$video_image = $s_img;
        $description = $s_desc;
        $tmp = explode('<description>', $doc);
        if (count($tmp) > 1)
        {
            $tmp = strstr($tmp[1], '</description>', true);
            $description = str_remove_spec($tmp);
			$description = strip_tags($description);
       }
		// ////////-//hd_print("--->>> $description");
        $year = '';
        $country = '';
        $genre = '';
		$duration_min = 0;
		$duration = '';
        $director = '';
        $actors = '';
        $kp_rating = '';
		$imdb_rating = '';
		$name_original = '';		
        // number of series
        $n_series = 2;
		}

        return  HD::encode_user_data
                (
                    array
                    (
                        'video_id' => $season_ref,
                        'n_series' => $n_series,
                        'page_ref' => $season_ref,
                        'video_image_url' => $video_image,
                        'title' => $video_title,
                        'genre' => str_remove_spec($genre),
                        'country' => str_remove_spec($country),
                        'year' => str_remove_spec($year),
                        'duration' => str_remove_spec($duration),
                        'duration_min' => $duration_min,
                        'director' => str_remove_spec($director),
                        'actors' => str_remove_spec($actors),
                        'description' => str_remove_spec($description),
                        'kp_rating' => $kp_rating,
						'imdb_rating' => $imdb_rating,
						'name_original' => $name_original,
                    )
                );
    }
    ///////////////////////////////////////////////////////////////////////

  
    private function parse_season($season_ref, &$plugin_cookies)
		{	
			$tmp = explode('&sinf&', $season_ref);
			$s_ref = $tmp[0];
			$s_titl = $tmp[1];
			$s_img = $tmp[2];
			$s_desc = $tmp[3];
			$items = array();
			if (preg_match("/http:\/\/vk.com\/video_ext.php/i",$s_ref)||preg_match("/youtube.com\/watch/i",$s_ref)){
			$video_title = $s_titl;
			$video_image = $s_img;
			$description = $s_desc;}
			elseif (preg_match("/m3u/i",$s_ref)){
			$video_title = $s_titl;
			$description = $s_desc;
			$logo = $s_img;
			$ndx_series = 0;
			if (!($m3u_lines = file($s_ref, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES))) 
			hd_print("Error opening $m3u_file");
			foreach ($m3u_lines as $line) {
			$line = trim($line);	
			if (preg_match('/#EXTM3U/', strtoupper($line), $matches))
				continue;
			
			if (preg_match('/^#EXTINF:[^,]*,(.+)$/', $line, $matches)) {
				$caption = trim($matches[1]);

				if (preg_match('/^#EXTINF:.*logo=([^ ^,]+)/', $line, $matches)) {
					$logo = preg_replace('/\"/', '', $matches[1]);			
				}
				
				continue;
			}
			array_push
					(
						$items,
						HD::encode_user_data
						(
							array
							(
								'episode_title' => str_remove_spec($caption),
								'episode_ref' => str_remove_spec($line),
								'season_title' => $video_title,
								'season_ref' => str_remove_spec($season_ref),
								'ndx_series' => $ndx_series,
								'poster_url' => str_remove_spec($logo),
								'description' => str_remove_spec($description),
							)
						)
					);
					++$this->m_items_count;
					++$ndx_series;
		}
    	return $items;
		}
		else{
        $doc = HD::http_get_document($s_ref);
		$doc = str_replace('<![CDATA[','',$doc);
		$doc = str_replace(']]>','',$doc);
		$doc = preg_replace('/<title>Назад.*\n.*<\/playlist_url>/i', '', $doc);
		$doc = str_replace('<playlist_url>http://93.188.161.204/vod/iptv/tvstart.xml</playlist_url>','',$doc);
        $video_title = $s_titl;
		$video_image = $s_img;
	
        // description
        $description = $s_desc;
        $tmp = explode('<description>', $doc);
        if (count($tmp) > 1)
        {
            $tmp = strstr($tmp[1], '</description>', true);
            $description = str_remove_spec($tmp);
			$description = strip_tags($description);
		}
		}	
		$is_new = false;
		$n = 0;
        $ndx_series = 0;
		if (preg_match("/api.uakino.net/i",$s_ref )|| preg_match("/http:\/\/vk.com\/video_ext.php/i",$s_ref)||preg_match("/youtube.com\/watch/i",$s_ref)){

        $episode_ref = $s_ref;
		$episode_title  = $video_title;
               
			   if (isset($plugin_cookies->$season_ref))
            if (!ListUtil::is_in_list($plugin_cookies->$season_ref, $episode_ref))
            $is_new = true;
			
            //{ Viewed items
            $is_viewed = false;
                if (!$is_new)
                {
                    $vs = $season_ref . ".viewed";
                    if (isset($plugin_cookies->$vs))
                    {
                        $is_viewed = ListUtil::is_in_list($plugin_cookies->$vs, $episode_ref);
                    }
                }
				//} Viewed items
				array_push
                (
                    $items,
                    HD::encode_user_data
                    (
                        array
                        (
                            //{ Viewed items
                            'episode_title' => str_remove_spec(($is_new ? '[NEW] ' : ($is_viewed ? '[VIEWED] ' : '')) . $episode_title),
                            //} Viewed items
                            'episode_ref' => str_remove_spec($episode_ref),
                            'season_title' => $video_title,
                            'season_ref' => str_remove_spec($season_ref),
                            'ndx_series' => $ndx_series,
                            'poster_url' => str_remove_spec($video_image),
                            'description' => str_remove_spec($description),
                        )
                    )
                );
                ++$this->m_items_count;
                ++$ndx_series;

			return $items;
		}
		if (preg_match("/ytsearchpl.php/i",$s_ref )){

        $videos = explode('<title>', $doc);
        unset($videos[0]);
        $videos = array_values($videos);
		$episode_title = "";
        foreach($videos as $video)
            {	
				////-//hd_print("-->video:$video");
                $tmp = explode('<stream_url>', $video);
				if (count($tmp) < 2)
				continue;
				$episode_ref = strstr($tmp[1], '</stream_url>', true);
				if (count($tmp) < 2)
				continue;
                $episode_title = strstr($video, '</title>', true);
				////////-//hd_print("-->episode_title:$episode_title");
                
                if (isset($plugin_cookies->$season_ref))
            if (!ListUtil::is_in_list($plugin_cookies->$season_ref, $episode_ref))
            $is_new = true;
			
            //{ Viewed items
            $is_viewed = false;
                if (!$is_new)
                {
                    $vs = $season_ref . ".viewed";
                    if (isset($plugin_cookies->$vs))
                    {
                        $is_viewed = ListUtil::is_in_list($plugin_cookies->$vs, $episode_ref);
                    }
                }
				//} Viewed items
				array_push
                (
                    $items,
                    HD::encode_user_data
                    (
                        array
                        (
                            //{ Viewed items
                            'episode_title' => str_remove_spec(($is_new ? '[NEW] ' : ($is_viewed ? '[VIEWED] ' : '')) . $episode_title),
                            //} Viewed items
                            'episode_ref' => str_remove_spec($episode_ref),
                            'season_title' => $video_title,
                            'season_ref' => str_remove_spec($season_ref),
                            'ndx_series' => $ndx_series,
                            'poster_url' => str_remove_spec($video_image),
                            'description' => str_remove_spec($description),
                        )
                    )
                );
                ++$this->m_items_count;
                ++$ndx_series;
            }
		
		return $items;
		}
		if (preg_match("/kinoprosmotr.net/i",$s_ref)){
		$doc = iconv('windows-1251', 'UTF-8', $doc);
		// title
        $video_title = $s_titl;
		$tmp = explode('<span>Название:', $doc);
		if (count($tmp) > 1)
        {
        $video_title = str_remove_spec(strstr($tmp[1], '</li>', true));
		$video_title = strip_tags($video_title);
		}
        // image
        $video_image = $s_img;
		$tmp = explode('<p class="full_movie_poster_img"><img src="', $doc);
		if (count($tmp) > 1)
        {
        $video_image = 'http://kinoprosmotr.net' . str_remove_spec(strstr($tmp[1], '"', true));
		}

        // description
        $description = $s_desc;
        $tmp = explode('<dt itemprop="description">', $doc);
        if (count($tmp) > 1)
        {
            $tmp = strstr($tmp[1], '</dt>', true);
            $description = str_remove_spec($tmp);
			$description = strip_tags($description);
		}
        // ////-//hd_print("--->>> $description");
		if (preg_match("/&amp;pl=/i",$doc)){
		$tmp = explode('&amp;pl=', $doc);
		if (count($tmp) > 1)
        $pl_link = urldecode (strstr($tmp[1], '&amp;', true));
		
		$doc_pl = HD::http_get_document($pl_link);

		$videos = explode('"file":"', $doc_pl);
        unset($videos[0]);
        $videos = array_values($videos);
		$episode_title = "";
        foreach($videos as $video)
            {	
                
                $episode_ref = strstr($video, '"', true);
				$tmp = explode('/', $episode_ref);
				$episode_title  = array_pop($tmp);
                if (isset($plugin_cookies->$season_ref))
            if (!ListUtil::is_in_list($plugin_cookies->$season_ref, $episode_ref))
            $is_new = true;
			
            //{ Viewed items
            $is_viewed = false;
                if (!$is_new)
                {
                    $vs = $season_ref . ".viewed";
                    if (isset($plugin_cookies->$vs))
                    {
                        $is_viewed = ListUtil::is_in_list($plugin_cookies->$vs, $episode_ref);
                    }
                }
				//} Viewed items
				array_push
                (
                    $items,
                    HD::encode_user_data
                    (
                        array
                        (
                            //{ Viewed items
                            'episode_title' => str_remove_spec(($is_new ? '[NEW] ' : ($is_viewed ? '[VIEWED] ' : '')) . $episode_title),
                            //} Viewed items
                            'episode_ref' => str_remove_spec($episode_ref),
                            'season_title' => $video_title,
                            'season_ref' => str_remove_spec($season_ref),
                            'ndx_series' => $ndx_series,
                            'poster_url' => str_remove_spec($video_image),
                            'description' => str_remove_spec($description),
                        )
                    )
                );
                ++$this->m_items_count;
                ++$ndx_series;
            }
			return $items;
		}
		else {
		$videos = explode(';file=', $doc);
        unset($videos[0]);
        $videos = array_values($videos);
		$episode_title = "";
        foreach($videos as $video)
            {	
                $episode_title = $video_title;
                $episode_ref = strstr($video, '&amp;', true);                
                if (isset($plugin_cookies->$season_ref))
            if (!ListUtil::is_in_list($plugin_cookies->$season_ref, $episode_ref))
            $is_new = true;
			
            //{ Viewed items
            $is_viewed = false;
                if (!$is_new)
                {
                    $vs = $season_ref . ".viewed";
                    if (isset($plugin_cookies->$vs))
                    {
                        $is_viewed = ListUtil::is_in_list($plugin_cookies->$vs, $episode_ref);
                    }
                }
				//} Viewed items
				array_push
                (
                    $items,
                    HD::encode_user_data
                    (
                        array
                        (
                            //{ Viewed items
                            'episode_title' => str_remove_spec(($is_new ? '[NEW] ' : ($is_viewed ? '[VIEWED] ' : '')) . $episode_title),
                            //} Viewed items
                            'episode_ref' => str_remove_spec($episode_ref),
                            'season_title' => $video_title,
                            'season_ref' => str_remove_spec($season_ref),
                            'ndx_series' => $ndx_series,
                            'poster_url' => str_remove_spec($video_image),
                            'description' => str_remove_spec($description),
                        )
                    )
                );
                ++$this->m_items_count;
                ++$ndx_series;
            }
			return $items;}
		}
        elseif (preg_match("/playlist_url/i",$doc))
			{	//////-//hd_print("-->doc:$doc");
				$playlist_urls = explode('<playlist_url>', $doc);
				$playlist_urls = array_values($playlist_urls);
				unset($playlist_urls[0]);
				foreach($playlist_urls as $playlist_url)
				{
					$playlist_link = strstr($playlist_url, '</playlist_url>', true);
					//////-//hd_print("-->playlist_link:$playlist_link");
					$playlist_doc = HD::http_get_document($playlist_link);
					$playlist_doc = explode('<title>Назад</title>',$playlist_doc);
					$playlist_doc = $playlist_doc[0];
					$playlist_doc = str_replace('<![CDATA[','',$playlist_doc);
					$playlist_doc = str_replace(']]>','',$playlist_doc);
					//////-//hd_print("-->playlist_doc:$playlist_doc");
						if (preg_match("/playlist_url/i",$playlist_doc))
						{
							$playlist_urls2 = explode('<playlist_url>', $playlist_doc);
							$playlist_urls2 = array_values($playlist_urls2);
							unset($playlist_urls2[0]);
							foreach($playlist_urls2 as $playlist_url2)
							{
							$playlist_link2 = strstr($playlist_url2, '</playlist_url>', true);
							$playlist_doc2 = HD::http_get_document($playlist_link2);
							$playlist_doc2 = str_replace('<title>Назад</title>','',$playlist_doc2);
							$playlist_doc2 = str_replace('<![CDATA[','',$playlist_doc2);
							$playlist_doc2 = str_replace(']]>','',$playlist_doc2);
							$videos = explode('<title>', $playlist_doc2);
							unset($videos[0]);
							$videos = array_values($videos);
							$episode_title = "";
							foreach($videos as $video)
								{	
									////////-//hd_print("-->video:$video");
									$tmp = explode('<stream_url>', $video);
									$episode_ref = strstr($tmp[1], '</stream_url>', true);
									if ($episode_ref === false)
									continue;
									$episode_title = strstr($video, '</title>', true);
									////////-//hd_print("-->episode_title:$episode_title");
									
									if (isset($plugin_cookies->$season_ref))
								if (!ListUtil::is_in_list($plugin_cookies->$season_ref, $episode_ref))
								$is_new = true;
								
								//{ Viewed items
								$is_viewed = false;
									if (!$is_new)
									{
										$vs = $season_ref . ".viewed";
										if (isset($plugin_cookies->$vs))
										{
											$is_viewed = ListUtil::is_in_list($plugin_cookies->$vs, $episode_ref);
										}
									}
									//} Viewed items
									array_push
									(
										$items,
										HD::encode_user_data
										(
											array
											(
												//{ Viewed items
												'episode_title' => str_remove_spec(($is_new ? '[NEW] ' : ($is_viewed ? '[VIEWED] ' : '')) . $episode_title),
												//} Viewed items
												'episode_ref' => str_remove_spec($episode_ref),
												'season_title' => $video_title,
												'season_ref' => str_remove_spec($season_ref),
												'ndx_series' => $ndx_series,
												'poster_url' => str_remove_spec($video_image),
												'description' => str_remove_spec($description),
											)
										)
									);
									++$this->m_items_count;
									++$ndx_series;
								}
							}
						}
						else
						{
						$videos = explode('<title>', $playlist_doc);
						unset($videos[0]);
						$videos = array_values($videos);
						$episode_title = "";
						foreach($videos as $video)
							{	
								////////-//hd_print("-->video:$video");
								$tmp = explode('<stream_url>', $video);
								if (count($tmp) < 2)
								continue;
								$episode_ref = strstr($tmp[1], '</stream_url>', true);
								////////-//hd_print("-->episode_ref:$episode_ref");
								$episode_title = strstr($video, '</title>', true);
								////////-//hd_print("-->episode_title:$episode_title");
								
								if (isset($plugin_cookies->$season_ref))
							if (!ListUtil::is_in_list($plugin_cookies->$season_ref, $episode_ref))
							$is_new = true;
							
							//{ Viewed items
							$is_viewed = false;
								if (!$is_new)
								{
									$vs = $season_ref . ".viewed";
									if (isset($plugin_cookies->$vs))
									{
										$is_viewed = ListUtil::is_in_list($plugin_cookies->$vs, $episode_ref);
									}
								}
								//} Viewed items
								array_push
								(
									$items,
									HD::encode_user_data
									(
										array
										(
											//{ Viewed items
											'episode_title' => str_remove_spec(($is_new ? '[NEW] ' : ($is_viewed ? '[VIEWED] ' : '')) . $episode_title),
											//} Viewed items
											'episode_ref' => str_remove_spec($episode_ref),
											'season_title' => $video_title,
											'season_ref' => str_remove_spec($season_ref),
											'ndx_series' => $ndx_series,
											'poster_url' => str_remove_spec($video_image),
											'description' => str_remove_spec($description),
										)
									)
								);
								++$this->m_items_count;
								++$ndx_series;
							}
						}
				}
			}

		$videos = explode('<title>', $doc);
        unset($videos[0]);
        $videos = array_values($videos);
		$episode_title = "";
        foreach($videos as $video)
            {	
				////-//hd_print("-->video:$video");
                $tmp = explode('<stream_url>', $video);
				if (count($tmp) < 2)
				continue;
				$episode_ref = strstr($tmp[1], '</stream_url>', true);
				if (count($tmp) < 2)
				continue;
                $episode_title = strstr($video, '</title>', true);
				////////-//hd_print("-->episode_title:$episode_title");
                
                if (isset($plugin_cookies->$season_ref))
            if (!ListUtil::is_in_list($plugin_cookies->$season_ref, $episode_ref))
            $is_new = true;
			
            //{ Viewed items
            $is_viewed = false;
                if (!$is_new)
                {
                    $vs = $season_ref . ".viewed";
                    if (isset($plugin_cookies->$vs))
                    {
                        $is_viewed = ListUtil::is_in_list($plugin_cookies->$vs, $episode_ref);
                    }
                }
				//} Viewed items
				array_push
                (
                    $items,
                    HD::encode_user_data
                    (
                        array
                        (
                            //{ Viewed items
                            'episode_title' => str_remove_spec(($is_new ? '[NEW] ' : ($is_viewed ? '[VIEWED] ' : '')) . $episode_title),
                            //} Viewed items
                            'episode_ref' => str_remove_spec($episode_ref),
                            'season_title' => $video_title,
                            'season_ref' => str_remove_spec($season_ref),
                            'ndx_series' => $ndx_series,
                            'poster_url' => str_remove_spec($video_image),
                            'description' => str_remove_spec($description),
                        )
                    )
                );
                ++$this->m_items_count;
                ++$ndx_series;
            }
		
		return $items;
		
    }
    ///////////////////////////////////////////////////////////////////////

    public function get_regular_folder_items($media_url, $from_ndx, &$plugin_cookies)
    {
	    //-//hd_print("--> get_regular_folder_items(): starting." . $media_url);
        if 
	    (
		    $media_url === 'main_menu' ||
		    $media_url === 'main_menu:videos' ||
		    $media_url === 'main_menu:search'
	    )
        {
            return $this->get_main_menu_items($media_url, $plugin_cookies);
        }

        if (intval($from_ndx) > 0)
            --$from_ndx; 
            
	    $videos = false;
        $folder_items = array();
        $total_num = 0;

        HD::decode_user_data($media_url, $media_str, $user_data);
		if (HD::has_attribute($user_data, 'category_ref'))
        {
            $videos = $this->parse_category($user_data, $plugin_cookies);
            $total_num = 0;
                
            if ($videos === false)
            {
				//-//hd_print("--> get_regular_folder_items(): no more available.");
                $caption = (is_interface_language_russian_or_similar() ? 'Больше ничего не найдено' : 'No more items found');
                $m_url = 'dummy';
                if (HD::has_attribute($user_data, 'fav'))
                {
                    $caption = (is_interface_language_russian_or_similar() ? 'Hе найдено' : 'Not found');
                    $m_url = $media_url;
                }
                $more_items_available = 0;
            

                $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
                array_push
                (
                                $folder_items,
                                array
                    (
                                    PluginRegularFolderItem::media_url => $m_url,
                                    PluginRegularFolderItem::caption => $caption,
                                    PluginRegularFolderItem::view_item_params => array
                                            (
                                                ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                                                ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                                                ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai'),
                                                ViewItemParams::item_detailed_icon_path => 'missing://'
                                            )
                    )
                );
                ++$total_num;
            }
            else
            {
				//-//hd_print("--> get_regular_folder_items(): items count: " . count($videos));
                $more_items_available = 0;

                foreach ($videos as $v)
                {
                    HD::decode_user_data($v, $media_str, $user_data);

                    array_push
                    (
                        $folder_items, 
                        array
                        (
                            PluginRegularFolderItem::media_url => $v,
                            PluginRegularFolderItem::caption => $user_data->site_title,
                            PluginRegularFolderItem::view_item_params => self::$catalog_vip
                        )
                     );
            
                    ++$total_num;
                }
            }
            return array(
                PluginRegularFolderRange::total => $total_num,
                PluginRegularFolderRange::count => count($folder_items),
                PluginRegularFolderRange::more_items_available => $more_items_available,
                PluginRegularFolderRange::from_ndx => $from_ndx,
                PluginRegularFolderRange::items => $folder_items
            );
        }
		else if (HD::has_attribute($user_data, 'site_ref'))
        {
            $videos = $this->parse_site($user_data);
            $total_num = 0;
                
            if ($videos === false)
            {
				//-//hd_print("--> get_regular_folder_items(): no more available.");
                $caption = (is_interface_language_russian_or_similar() ? 'Больше ничего не найдено' : 'No more items found');
                $m_url = 'dummy';
                if (HD::has_attribute($user_data, 'fav'))
                {
                    $caption = (is_interface_language_russian_or_similar() ? 'Hе найдено' : 'Not found');
                    $m_url = $media_url;
                }
                $more_items_available = 0;
            

                $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
                array_push
                (
                                $folder_items,
                                array
                    (
                                    PluginRegularFolderItem::media_url => $m_url,
                                    PluginRegularFolderItem::caption => $caption,
                                    PluginRegularFolderItem::view_item_params => array
                                            (
                                                ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                                                ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                                                ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai'),
                                                ViewItemParams::item_detailed_icon_path => 'missing://'
                                            )
                    )
                );
                ++$total_num;
            }
            else
            {
				//-//hd_print("--> get_regular_folder_items(): items count: " . count($videos));
                $more_items_available = 0;

                foreach ($videos as $v)
                {
                    HD::decode_user_data($v, $media_str, $user_data);

                    array_push
                    (
                        $folder_items, 
                        array
                        (
                            PluginRegularFolderItem::media_url => $v,
                            PluginRegularFolderItem::caption => $user_data->subcategory_title,
                            PluginRegularFolderItem::view_item_params => self::$catalog_vip
                        )
                     );
            
                    ++$total_num;
                }
            }
            return array(
                PluginRegularFolderRange::total => $total_num,
                PluginRegularFolderRange::count => count($folder_items),
                PluginRegularFolderRange::more_items_available => $more_items_available,
                PluginRegularFolderRange::from_ndx => $from_ndx,
                PluginRegularFolderRange::items => $folder_items
            );
        }
        else if (HD::has_attribute($user_data, 'season_ref'))
        {
            if (HD::has_attribute($user_data, 'season_ref'))
                $season_ref = $user_data->season_ref;
            else if (HD::has_attribute($user_data, 'fav'))
                $season_ref = $user_data->fav;
            $videos = $this->parse_season($season_ref, $plugin_cookies);
            $total_num = 0;
                
           if ($videos === false)
            {
				//-//hd_print("--> get_regular_folder_items(): no more available.");
                $caption = (is_interface_language_russian_or_similar() ? 'Больше ничего не найдено' : 'No more items found');
                $m_url = 'dummy';
                if (HD::has_attribute($user_data, 'fav'))
                {
                    $caption = (is_interface_language_russian_or_similar() ? 'Hе найдено' : 'Not found');
                    $m_url = $media_url;
                }
		        $more_items_available = 0;
                
                if ($media_url === 'main_menu:fav')
                {
                    $folder->set_expired();
                }

                $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
		        array_push
		        (
                    $folder_items,
                    array
			        (
                        PluginRegularFolderItem::media_url => $m_url,
	                    PluginRegularFolderItem::caption => $caption,
                	    PluginRegularFolderItem::view_item_params => array
                        	    (
                                    ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                                    ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                                    ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai'),
                		            ViewItemParams::item_detailed_icon_path => 'missing://'
                        		)
			        )
		        );
                ++$total_num;
                
                $this->fav_build_in_progess = false;
            }
            else
            {
                //-//hd_print("--> get_regular_folder_items(): items count: " . count($videos));
                $more_items_available = 0;

                foreach ($videos as $v)
                {
                    HD::decode_user_data($v, $media_str, $user_data);

                    array_push
                    (
                        $folder_items, 
                        array
                        (
                            PluginRegularFolderItem::media_url => $v,
                            PluginRegularFolderItem::caption => $user_data->episode_title,
                            PluginRegularFolderItem::view_item_params => array(
                                        ViewItemParams::item_paint_icon => true,
                                        ViewItemParams::icon_path => 'gui_skin://small_icons/movie.aai',
                                        ViewItemParams::item_layout => HALIGN_LEFT,
                                        ViewItemParams::icon_valign => VALIGN_CENTER,
                                        ViewItemParams::icon_dx => 20,
                                        ViewItemParams::icon_dy => -5,
                                        ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
                                        ViewItemParams::item_caption_width => 1550
                            ),
                        )
                     );
            
                    ++$total_num;
                }
            }
            return array(
                PluginRegularFolderRange::total => $total_num,
                PluginRegularFolderRange::count => count($folder_items),
                PluginRegularFolderRange::more_items_available => $more_items_available,
                PluginRegularFolderRange::from_ndx => $from_ndx,
                PluginRegularFolderRange::items => $folder_items
            );
        }
        
        $folder = FolderCache::get($media_url);

        if (HD::has_attribute($user_data, 'category_ref'))
        {
            $videos = $folder->get_elements($from_ndx, $plugin_cookies);
            $total_num = 0;
                
            if ($videos === false)
            {
		        //-//hd_print("--> get_regular_folder_items(): no more available.");
                $caption = (is_interface_language_russian_or_similar() ? 'Больше ничего не найдено' : 'No more items found');
                $m_url = 'dummy';
                if (HD::has_attribute($user_data, 'fav'))
                {
                    $caption = (is_interface_language_russian_or_similar() ? 'Hе найдено' : 'Not found');
                    $m_url = $media_url;
                }
		        $more_items_available = 0;
                
                if ($media_url === 'main_menu:fav')
                {
                    $folder->set_expired();
                }

                $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
		        array_push
		        (
                    $folder_items,
                    array
			        (
                        PluginRegularFolderItem::media_url => $m_url,
	                    PluginRegularFolderItem::caption => $caption,
                	    PluginRegularFolderItem::view_item_params => array
                        	    (
                                    ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                                    ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                                    ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai'),
                		            ViewItemParams::item_detailed_icon_path => 'missing://'
                        		)
			        )
		        );
                ++$total_num;
                
                $this->fav_build_in_progess = false;
            }
            else
            {
                //-//hd_print("--> get_regular_folder_items(): items count: " . count($videos));
                $more_items_available = 1;

                foreach ($videos as $v)
                {
                    HD::decode_user_data($v, $media_str, $user_data);

                    array_push
                    (
                        $folder_items, 
                        array
                        (
                            PluginRegularFolderItem::media_url => $v,
                            PluginRegularFolderItem::caption => $user_data->subcategory_title,
                            PluginRegularFolderItem::view_item_params => self::$catalog_vip
                        )
                     );
            
                    ++$total_num;
                }
            }
            return array(
                PluginRegularFolderRange::total => $total_num,
                PluginRegularFolderRange::count => count($folder_items),
                PluginRegularFolderRange::more_items_available => $more_items_available,
                PluginRegularFolderRange::from_ndx => $from_ndx,
                PluginRegularFolderRange::items => $folder_items
            );
        }
        else if
        (
            HD::has_attribute($user_data, 'subcategory_ref') ||
            HD::has_attribute($user_data, 'ser_feed') ||
            $media_url === 'main_menu:fav'
        )
        {
       	    $videos = $folder->get_elements($from_ndx, $plugin_cookies);
            $total_num = $folder->get_num_elements();
	            
           if ($videos === false)
            {
		        //-//hd_print("--> get_regular_folder_items(): no more available.");
                $caption = (is_interface_language_russian_or_similar() ? 'Больше ничего не найдено' : 'No more items found');
                $m_url = 'dummy';
                if (HD::has_attribute($user_data, 'fav'))
                {
                    $caption = (is_interface_language_russian_or_similar() ? 'Hе найдено' : 'Not found');
                    $m_url = $media_url;
                }
		        $more_items_available = 0;
                
                if ($media_url === 'main_menu:fav')
                {
                    $folder->set_expired();
                }

                $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
		        array_push
		        (
                    $folder_items,
                    array
			        (
                        PluginRegularFolderItem::media_url => $m_url,
	                    PluginRegularFolderItem::caption => $caption,
                	    PluginRegularFolderItem::view_item_params => array
                        	    (
                                    ViewItemParams::icon_width => ($view_type == 1 ? 190 : ($view_type == 2 ? 150 : 50)),
                                    ViewItemParams::icon_height => ($view_type == 1 ? 290 : ($view_type == 2 ? 200 : 50)),
                                    ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai'),
                		            ViewItemParams::item_detailed_icon_path => 'missing://'
                        		)
			        )
		        );
                ++$total_num;
                
                $this->fav_build_in_progess = false;
            }
            else
            {
	            //-//hd_print("--> get_regular_folder_items(): subcategory_ref: items count: " . count($videos));
                if ($media_url === 'main_menu:fav')
                    $this->fav_build_in_progess = true;
                else
                    $this->fav_build_in_progess = false;

                $more_items_available = 1;

                foreach ($videos as $v)
                {
	                HD::decode_user_data($v, $media_str, $user_data);
		    
		            $genre = ' ';
		            if (HD::has_attribute($user_data, 'genre'))
			            $genre = $user_data->genre;

                    $duration = ' ';
		            if (HD::has_attribute($user_data, 'duration'))
			            $duration = $user_data->duration;

                    if (is_interface_language_russian_or_similar())
                    {
		                $genre_caption = 'Жанр';
                        $duration_caption = 'Продолжительность';
                    }
                    else
                    {
		                $genre_caption = 'Genre';
                        $duration_caption = 'Duration';
                    }

                    $info = '';
                    $title = ' ';
                    // Workaround for a bug in Shell.
                    if (HD::has_attribute($user_data, 'season_title'))
                        $title = str_replace('^', '', $user_data->season_title);
                                
                    $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;

                    //$info =
		            //    $genre_caption . ': | ' . $genre . ' || ' .
	                //        $duration_caption . ': | ' . $duration . ' || ';

		            $video_id = '1';
		            $n_series = 1;
		            $ndx_series = 0;
                    $is_has_new = false;

		            if (HD::has_attribute($user_data, 'video_id'))
	                    $video_id = $user_data->video_id;

 		            if (HD::has_attribute($user_data, 'n_series'))
	                    $n_series = $user_data->n_series;

                    if (HD::has_attribute($user_data, 'is_has_new'))
                        $is_has_new = $user_data->is_has_new;

                    if ($view_type == 1)
                    {
                            array_push(
                                $folder_items, 
                                array(
                                    PluginRegularFolderItem::media_url =>
                                        HD::encode_user_data(
                                            array(
				                	            'video_id' => $video_id,
                                                'n_series' => $n_series,
                                                'season_title' => $title
				                            )
			                        ),
                                    PluginRegularFolderItem::caption => ($is_has_new ? '[NEW] ' . $title : $title),
                                    PluginRegularFolderItem::view_item_params => array(
                                        ViewItemParams::icon_path => $this->get_preview_url($user_data),
                                        ViewItemParams::item_paint_caption => false,
                                        ViewItemParams::item_caption_wrap_enabled => TRUE,
            //                            ViewItemParams::item_detailed_info => $info
                                    )
                                ));
                    }
                    else if ($view_type == 2)
                    {
                            array_push(
                                $folder_items, 
                                array(
                                    PluginRegularFolderItem::media_url =>
                                        HD::encode_user_data(
                                            array(
                                                'video_id' => $video_id,
                                                'n_series' => $n_series,
                                            )
                                    ),
                                    PluginRegularFolderItem::caption => ($is_has_new ? '[NEW] ' . $title : $title),
                                    PluginRegularFolderItem::view_item_params => array(
                                        ViewItemParams::icon_path => $this->get_preview_url($user_data),
                                        ViewItemParams::item_detailed_info => $info
                                    )
                                ));
                    }
                    else
                    {
                        array_push(
                            $folder_items, 
                            array(
                                PluginRegularFolderItem::media_url =>
                                    HD::encode_user_data(
                                        array(
                                            'video_id' => $video_id,
                                            'n_series' => $n_series
                                        )
                                ),
                                PluginRegularFolderItem::caption => ($is_has_new ? '[NEW] ' . $title : $title),
                                PluginRegularFolderItem::view_item_params => array(
                                    ViewItemParams::item_detailed_icon_path => $this->get_preview_url($user_data),
                                    ViewItemParams::item_detailed_info => $info
                                )
                            ));
                    }
                }

                array_push(
                    $folder_items, 
                    array(
                        PluginRegularFolderItem::media_url => 'dummy',
                        PluginRegularFolderItem::caption =>
                            (is_interface_language_russian_or_similar() ? 'Загрузка...' : 'Loading...'),
                        PluginRegularFolderItem::view_item_params => array(
                            ViewItemParams::icon_width => ($view_type == 1 ? 60 : ($view_type == 2 ? 150 : 50)),
                            ViewItemParams::icon_height => (1 ? 60 : ($view_type == 2 ? 150 : 50)),
                            ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai'),
                            ViewItemParams::item_detailed_icon_path => 'missing://'
                        )
                    ));

                ++$total_num;
            }
        }
        else if (HD::has_attribute($user_data, 'video_id'))
        {
	        $more_items_available = 0;
            $c = VideoCache::get($user_data->video_id);
            HD::decode_user_data($c, $media_str, $user_data);
	        $vod_info = $this->get_vod_info($media_url, &$plugin_cookies);
	    
	        $series = $vod_info[PluginVodInfo::series];

	        $i = 0;
	        foreach($series as $s)
	        {
		        $info = '';
                $isnew = false;
                $video_id = $user_data->video_id;
                if (isset($plugin_cookies->$video_id))
                {
                    $isnew = !ListUtil::is_in_list($plugin_cookies->$video_id, $s[PluginVodSeriesInfo::playback_url]);
                }
				//{ Viewed items
                $is_viewed = false;
                if (!$isnew)
                {
                    $vs = $video_id . ".viewed";
                    if (isset($plugin_cookies->$vs))
                    {
                        $is_viewed = ListUtil::is_in_list($plugin_cookies->$vs, $s[PluginVodSeriesInfo::playback_url]);
                    }
                }
                //} Viewed items
                array_push
                (
                    $folder_items, 
                    array
                    (
                        PluginRegularFolderItem::media_url =>
                            HD::encode_user_data
                            (
                                array
                                (
					                'video_id' => $user_data->video_id,
					                'ndx_series' => $i,
                                    'url_series' => $s[PluginVodSeriesInfo::playback_url]
				                )
  			                ),
                            //{ Viewed items
                            PluginRegularFolderItem::caption => (
                                                                    $isnew ? "[NEW] " . $s[PluginVodSeriesInfo::name]  : ($is_viewed ? "[VIEWED] " . $s[PluginVodSeriesInfo::name] : $s[PluginVodSeriesInfo::name])
                                                                ),
                            //} Viewed items
                            PluginRegularFolderItem::view_item_params => array(
                                    ViewItemParams::icon_path => 'gui_skin://small_icons/movie.aai',
                                    ViewItemParams::item_detailed_info => $info
                            )
                    )
		        );
		        ++$i;
	        }

	        $total_num = $i;
        }
    

        return array(
            PluginRegularFolderRange::total => $total_num,
            PluginRegularFolderRange::count => count($folder_items),
            PluginRegularFolderRange::more_items_available => $more_items_available,
            PluginRegularFolderRange::from_ndx => $from_ndx,
            PluginRegularFolderRange::items => $folder_items
        );
    }

	private function get_episode_url_moonwalk($episode_ref, $video_quality)
	{
        $vquality = $this->get_quality_array(false);
        $q = $vquality[$video_quality];

        // get playlist with set of video for diff quality
        $doc = HD::http_get_document($episode_ref);
		preg_match("/post\('([^']*)', {[^']*video_token: '([^']*)'/i", $doc, $matches);////
	//	////////-//hd_print("test--->>> $matches[2]");

        $post_data = 'video_token='. $matches[2].'&stats_version=1&video_secret=HIV5';
        $urls = 'http://moonwalk.cc' . $matches[1];
        $doc = HD::http_post_document($urls, $post_data);
        preg_match_all('/"manifest_m3u8":"(.*?)"/', $doc, $manifest_m3u8);
		preg_match_all('/"manifest_f4m":"(.*?)"/', $doc, $manifest_f4m);
        $url = $manifest_m3u8[1][0];
        
        $a = explode('/', $url);
        $a[count($a) - 1] = '';
        $base_url = implode('/', $a);
        
        // get video for diff quality
        $doc = '';
        try
        {
            $doc = HD::http_get_document($url);
        }
        catch(Exception $e)
        {
            $doc = HD::http_get_document($url);
        }
        if (preg_match_all('/#EXT-X-STREAM-INF:[^,]*,RESOLUTION=[^x]*x([^,]*),BANDWIDTH=[^\n]*\n([^\n]*)/is', $doc, $matches))
        {
            $urls = array();
            $url = false;
            $qmin = 5000;
            $qmax = 0;
            $n = count($matches[0]);
            for ($i = 0; $i < $n; $i++)
            {
                if (intval($matches[1][$i]) < $qmin)
                    $qmin = intval($matches[1][$i]);
                if (intval($matches[1][$i]) > $qmax)
                    $qmax = intval($matches[1][$i]);
                if ($matches[1][$i] == $q)
                {
                    $url = $base_url . $matches[2][$i];
                    break;
                }
                $urls[$matches[1][$i]] = $matches[2][$i];
            }
            
            if (!$url)
            {
                if ($q == '240')
                    $url = $base_url . $urls[strval($qmin)];
                else if ($q == '1080')
                    $url = $base_url . $urls[strval($qmax)];
                else
                    $url = $base_url . $matches[2][0];
            }
        }
		
    else
        {
        if (file_exists('/persistfs/codecpack/WWW/cgi-bin/hls_moonwalk.sh')){
		$command = '/persistfs/codecpack/WWW/cgi-bin/hls_moonwalk.sh' .' ' . $manifest_m3u8[1][0] .' ' . $video_quality;
		}
			if (!file_exists('/persistfs/codecpack/WWW/cgi-bin/hls_moonwalk.sh')) 
			{
			if (file_exists('/D/codecpack/WWW/cgi-bin/hls_moonwalk.sh'))
				$command = '/D/codecpack/WWW/cgi-bin/hls_moonwalk.sh' .' ' . $manifest_m3u8[1][0] .' ' . $video_quality;
			}
		   shell_exec($command);
		   if (file_exists('/tmp/www/play.m3u8')){
		        $url = "http://127.0.0.1/play.m3u8";
			}
			// ////////-//hd_print('else url------------->' . $manifest_m3u8[1][0]);
        }
		$url = str_replace("http://37.220.36.28", "http://37.220.36.26", $url);
		$url = str_replace("http://37.220.36.11", "http://37.220.36.26", $url);
        ////////-//hd_print("--->>> moonwalk url: $url");
        return $url;
    }
    
	///////////////////////////////////////////////////////////////////////
    
	private function get_episode_url($media_url, $video_quality)
    {	
		$media_url = str_replace("https:", "http:", $media_url);
		$media_url = str_replace("&amp;", "&", $media_url);
		$media_url = str_replace("youtube.com/embed/", "youtube.com/watch?v=", $media_url);
		$media_url = str_replace("youtu.be/", "youtube.com/watch?v=", $media_url);
		$media_url = str_replace("youtube.com/watch?feature=player_embedded&v=", "youtube.com/watch?v=", $media_url);
		$media_url = str_replace("www.youtube-nocookie.com/v/", "youtube.com/watch?v=", $media_url);
		$media_url = str_replace("youtube.com/v/", "youtube.com/watch?v=", $media_url);
		$media_url = str_replace("vkontakte.ru", "vk.com", $media_url);
		$media_url = str_replace("http://37.220.36.28", "http://37.220.36.26", $media_url);
		$media_url = str_replace("http://37.220.36.11", "http://37.220.36.26", $media_url);
        $tmp = explode('&season_ref=', $media_url);
        $media_url = $tmp[0];
		//-//hd_print("-->media_url:$media_url");
        if (preg_match("/www.youtube.com/i",$media_url))
        {
			$tmp = explode("watch?v=", $media_url);
			$url = $tmp[1];
			$url = Youtube::retrieve_playback_url($url, $plugin_cookies);
		}
		elseif (preg_match("/vimeo.com/i",$media_url))
        {
			$url = Vimeo::retrieve_playback_url($media_url, $plugin_cookies);
			
			
		}
		elseif (preg_match("/rutube/i",$media_url))
			{ 
			if (preg_match("/video\/embed/i",$media_url))
				{
				$doc = HD::http_get_document($media_url);
				preg_match_all('/<link rel="canonical" href="(.*?)"/', $doc, $tmp);
				$media_url = $tmp[1][0];
				}
			elseif (preg_match("/\/\/video.rutube.ru/i",$media_url))
				{
				$media_url = str_replace("//video.rutube.ru/", "//rutube.ru/video/embed/", $media_url);
				$doc = HD::http_get_document($media_url);
				preg_match_all('/<link rel="canonical" href="(.*?)"/', $doc, $tmp);
				$media_url = $tmp[1][0];
				}
			$url = (str_replace("video", 'api/play/trackinfo', $media_url)). '?format=xml';
			$doc = HD::http_get_document($url);
			preg_match_all('/<m3u8>(.*?)<\/m3u8>/', $doc, $tmp);
			$url = $tmp[1][0];
			}
        elseif (preg_match("/vk.com/i",$media_url))
		{ 
			$doc = HD::http_get_document($media_url);
			if (preg_match("/video_no_flv = /i",$doc))
			{
			$tmp = explode("video_no_flv = ", $doc);
			$is_not_flv = intval(strstr($tmp[1], "'", true));
			$tmp = explode("var video_host = '", $doc);
			$url = strstr($tmp[1], "'", true);
			
			if (strstr($url, "http://") == false)
				$url = "http://mp4://" . $url . "/";
			else
				$url = str_ireplace("http://", "http://mp4://", $url);
			
			if ($is_not_flv)
			{
				$tmp = explode("var video_uid = '", $doc);
				$url = $url . 'u' . strstr($tmp[1], "'", true) . '/videos/';

				$tmp = explode("var video_vtag = '", $doc);
				$url = $url . strstr($tmp[1], "'", true) . '.';

				$vquality = $this->get_quality_array(false);
				$tmp = explode("var video_max_hd = '", $doc);
				$vq_max = intval(strstr($tmp[1], "'", true));
				
				if ($video_quality > $vq_max)
					$video_quality = $vq_max;
					
				$url = $url . $vquality[$video_quality] . '.mp4';
			}
			else
			{
				$url = $url . 'assets/videos/';
				$tmp = explode("vtag=", $doc);
				$url = $url . strstr($tmp[1], "&", true);
				$tmp = explode("vkid=", $doc);
				$url = $url . strstr($tmp[1], "&", true);
				$url = $url . '.vk.flv';
				$url = str_ireplace("mp4://", "", $url);
			}}
			elseif (preg_match("/rutube.ru/i",$doc))
			{
			preg_match_all('/" src="\/\/(.*?)\?/', $doc, $tmp);
			$media_url = $tmp[1][0];
			if (!preg_match("/http/i",$media_url))
			$media_url = 'http://' . $media_url;
			if (preg_match("/video\/embed/i",$media_url))
				{
				$doc = HD::http_get_document($media_url);
				preg_match_all('/<link rel="canonical" href="(.*?)"/', $doc, $tmp);
				$media_url = $tmp[1][0];
				}
			elseif (preg_match("/https:\/\/video.rutube.ru/i",$media_url))
				{
				$media_url = str_replace("https://video.rutube.ru/", "http://rutube.ru/video/embed/", $media_url);
				$doc = HD::http_get_document($media_url);
				preg_match_all('/<link rel="canonical" href="(.*?)"/', $doc, $tmp);
				$media_url = $tmp[1][0];
				}
			$url = (str_replace("video", 'api/play/options', $media_url)). '?format=xml';
			$doc = HD::http_get_document($url);
			preg_match_all('/<m3u8>(.*?)<\/m3u8>/', $doc, $tmp);
			$url = $tmp[1][0]. '|||protocol:hls';
			}
			else
			{
			$tmp = explode("php?", $media_url);
			$media_url = 'http://api.vk.com/method/video.getEmbed?'. $tmp[1];
			$media_url = str_replace('&id=', "&video_id=", $media_url);
			$media_url = str_replace('hash=', "embed_hash=", $media_url);
			$doc = HD::http_get_document($media_url);
			$doc = str_replace('\\', "", $doc);
			$tmp = explode("no_flv\":", $doc);
			$is_not_flv = intval(strstr($tmp[1], ",", true));
			$tmp = explode('"host":"', $doc);
			$url = strstr($tmp[1], '"', true);
			
			if (strstr($url, "http://") == false)
				$url = "http://mp4://" . $url . "/";
			else
				$url = str_ireplace("http://", "http://mp4://", $url);
			
			if ($is_not_flv)
			{
				$tmp = explode('"uid":"', $doc);
				$url = $url . 'u' . strstr($tmp[1], '"', true) . '/videos/';

				$tmp = explode('"vtag":"', $doc);
				$url = $url . strstr($tmp[1], '"', true) . '.';

				$vquality = $this->get_quality_array(false);
				$tmp = explode('"hd":', $doc);
				$vq_max = intval(strstr($tmp[1], ".", true));
				
				if ($video_quality > $vq_max)
					$video_quality = $vq_max;
					
				$url = $url . $vquality[$video_quality] . '.mp4';
			}
			else
			{
				$url = $url . 'assets/videos/';
				$tmp = explode("vtag=", $doc);
				$url = $url . strstr($tmp[1], "&", true);
				$tmp = explode("vkid=", $doc);
				$url = $url . strstr($tmp[1], "&", true);
				$url = $url . '.vk.flv';
				$url = str_ireplace("mp4://", "", $url);
			}
			}
		} 
		elseif (preg_match("/moonwalk.cc/i",$media_url)){ 
                    return $this->get_episode_url_moonwalk($media_url, $video_quality);
				}
		elseif (preg_match("/37.220.36.26/i",$media_url)){
			if (preg_match("/tracks-3,4/i",$media_url)){
			$video_quality = '1';
			$media_url = str_replace('/tracks-3,4', "", $media_url);
			}
			if (preg_match("/tracks-2,4/i",$media_url)){
			$video_quality = '2';
			$media_url = str_replace('/tracks-2,4', "", $media_url);
			}
			if (preg_match("/tracks-1,4/i",$media_url)){
			$video_quality = '3';
			$media_url = str_replace('/tracks-1,4', "", $media_url);
			}
			
			if (file_exists('/persistfs/codecpack/WWW/cgi-bin/hls_moonwalk.sh')){
			$command = '/persistfs/codecpack/WWW/cgi-bin/hls_moonwalk.sh' .' ' . $media_url .' ' . $video_quality;
			}
			if (!file_exists('/persistfs/codecpack/WWW/cgi-bin/hls_moonwalk.sh')) 
			{
			if (file_exists('/D/codecpack/WWW/cgi-bin/hls_moonwalk.sh'))
				$command = '/D/codecpack/WWW/cgi-bin/hls_moonwalk.sh' .' ' . $media_url .' ' . $video_quality;
			}
		   shell_exec($command);
		   if (file_exists('/tmp/www/play.m3u8')){
		        $url = "http://127.0.0.1/play.m3u8";
			}
        }
		elseif (preg_match("/.kinoylei.ru/i",$media_url)){
		$doc = HD::http_get_document('http://server1.kinoylei.ru/get2/3074');
		preg_match('|film\\/(.*)\\/|isU',$doc,$md5hash);
		$url = str_replace('md5hash',$md5hash[1],$media_url);
		$url = str_replace('.flv','.mp4',$url);
		}
		elseif (preg_match("/stepashka.com/i",$media_url)){
		$doc = HD::http_get_document('http://online.stepashka.com/filmy/trillery/26171-oblivion-oblivion-2013.html');
		preg_match('|st=http://online.stepashka.com\\/player\\/26171\\/.*\\/(.*)\\/1|isU',$doc,$md5hash);
		$url = str_replace('md5hash',$md5hash[1],$media_url);
		}
		elseif (preg_match("/kaban.tv\/archive/i",$media_url)){
		$doc = HD::http_get_document($media_url);
		preg_match('|,"file":"(.*)","f|isU',$doc,$file);
		if($file[1] != False){
			$url = 'http://93.188.161.204/php/kaban/kaban.php?code='.$file[1];
			$url = HD::http_get_document($url);
			}
		}
		elseif (preg_match("/poiuytrew.pw/i",$media_url)){
		$doc = HD::http_get_document('http://filmix.net/dramy/82725-vnutri-lyuina-devisa-inside-llewyn-davis-2013.html');
		preg_match('|&amp;file=(.*)&amp;.*&amp;|isU',$doc,$file);
		if($file[1] != False){
			$url = 'http://93.188.161.204/php/filmix.php?code='.$file[1];
			$html = HD::http_get_document($url);
			$media_url = str_replace('md5hash',$html,$media_url);
			if(preg_match('|[720,480,360]|isU',$media_url)){
			$media_url = str_replace('[720,480,360]','720',$media_url);}
			elseif(preg_match('|[,480,360]|isU',$media_url)){
			$media_url = str_replace('[,480,360]','480',$media_url);}
			}
			else{
			$url = $media_url;
			}
		}
		elseif (preg_match("/pirateplayer.com/i",$media_url)){
			$doc = HD::http_get_document('http://online.stepashka.com/filmy/detektiv/29140-falshivaya-kukla-hamis-a-baba-1991.html');
			preg_match('|st=http://online.stepashka.com\\/player\\/.*\\/.*\\/(.*)\\/1|isU',$doc,$md5hash);
			$url = str_replace('md5hash',$md5hash[1],$media_url);
			}
		elseif (preg_match("/videokub.com/i",$media_url)){
			$doc = HD::http_get_document($media_url);
			preg_match('|file:"(.*)\\/|isU',$doc,$md5hash);
			$url = $md5hash[1];
			}
		elseif (preg_match("/hotcloud.org/i",$media_url)){
			$doc = HD::http_get_document('https://my-hit.org/film/558/playlist.txt');
			preg_match('|id=(.*)"}]}|isU',$doc,$md5hash);
			$url = str_replace('md5hash',$md5hash[1],$media_url);
			}
		elseif (preg_match("/moviestape.com/i",$media_url)){
			$doc = HD::http_get_document('http://fs0.moviestape.com/show.php?name=films/Captain.Phillips.mp4');
			preg_match("|file : 'http:\\/\\/.*\\/video\\/(.*)\\/.*\\/.*mp4'};|isU",$doc,$md5hash);
			$url = str_replace('md5hash',$md5hash[1],$media_url);
			}
		elseif (preg_match("/aburmu4.tv/i",$media_url)){
			$doc = HD::http_get_document('http://s1.aburmu4.tv/player/play.php?s=1&name=vsfw/antboy_2013.flv');
			preg_match("|file : 'http:\\/\\/.*\\/v\\/(.*)\\/.*\\/.*flv'};|isU",$doc,$md5hash);
			$media_url = str_replace('md5hash',$md5hash[1],$media_url);
			$url = str_replace('aburmu4.tv','',$media_url);
			}
		elseif (preg_match("/serialo.ru/i",$media_url)){
			$doc = HD::http_get_document('http://latino-serialo.ru/italianskie_seriali_online/2638-polny-ocharovaniya-cheias-de-charme-seriya-10.html');
			preg_match('|;pl=(.*)"|isU',$doc,$file);
			if($file[1] != False){
				$html = HD::http_get_document($file[1]);
				preg_match('|file":"(.*)"|isU',$html,$play);
				preg_match('|\\/video\\/(.*)\\/|isU',$play[1],$md5hash);
				$play2 = str_replace($md5hash[1],'md5hash',$play[1]);
				preg_match('|\\/md5hash\\/(.*)\\/|isU',$play2,$md4hash);
				$media_url = str_replace('md5hash',$md5hash[1],$media_url);
				$url = str_replace('md4hash',$md4hash[1],$media_url);
				}
			}
		elseif (preg_match("/divan.tv/i",$media_url)){
			$doc = HD::http_get_document($media_url);
			preg_match('|file: "(.*)",|isU',$doc,$md5hash);
			$url = $md5hash[1];
			}
		elseif (preg_match("/baskino.com/i",$media_url)){
			$doc = HD::http_get_document($media_url);
			preg_match('|file:"(.*)"|isU',$doc,$md5hash);
			$url = $md5hash[1];
			}
		elseif (preg_match("/kinobar.net/i",$media_url)){
			$doc = HD::http_get_document($media_url);
			preg_match('|<source src="(.*)" type=|isU',$doc,$md5hash);
			$url = $md5hash[1];
			}
		elseif (preg_match("/serials.tv/i",$media_url)){
			$doc = HD::http_get_document($media_url);
			preg_match("|src: '(.*)',|isU",$doc,$md5hash);
			$url = $md5hash[1];
			}
		elseif (preg_match("/jampo.tv/i",$media_url)){
			$doc = HD::http_get_document($media_url);
			preg_match('|<video width=".*" height=".*" src="(.*)" controls />|isU',$doc,$md5hash);
			$url = $md5hash[1];
			}
		elseif (preg_match("/lidertvvv/i",$media_url)){
			$media_url = str_replace('lidertvvv','',$media_url);
			$doc = HD::http_get_document($media_url);
			preg_match('|file: "(.*)",|isU',$doc,$md5hash);
			$url = $md5hash[1];
			}
		elseif (preg_match("/minizal.net/i",$media_url)){
			$doc = HD::http_get_document('http://s2.minizal.net/php/play.php?name=film/pod.krovatju.2012.bdrip.flv');
			preg_match('|video\\/(.*)\\/|isU',$doc,$md5hash);
			$url = str_replace('md5hash',$md5hash[1],$media_url);
			}
		elseif (preg_match("/kinoxa-x.ru/i",$media_url)){
			$doc = HD::http_get_document($media_url);
			preg_match('|<video.*src="(.*)" controls|isU',$doc,$md5hash);
			$url = $md5hash[1];
			}
		elseif (preg_match("/50.7.172.194/i",$media_url)){
			$doc = HD::http_get_document($media_url);
			preg_match('|<video.*src="(.*)" controls|isU',$doc,$md5hash);
			$url = $md5hash[1];
			}
		elseif (preg_match("/allserials.tv/i",$media_url)){
			$doc = HD::http_get_document('http://allserials.tv/get.php?action=playlist&pl=Osennie.cvety.2009');
			$doc = preg_replace('|\n|','',$doc);
			if($doc != False){
				$url = 'http://gegen-abzocke.com/xml/nstrim/allserials/code.php?code_url='.$doc;
				$html = HD::http_get_document($url);
				$url = str_replace('md5hash',$html,$media_url);
				}
			}
		elseif(preg_match("/igru-film.net/i",$media_url)){
			preg_match('|sa(.*)xyss(.*)xy|isU',$media_url,$url);
			$doc = HD::http_get_document('http://fepcom.net/'.$url[2]);
			preg_match('|file=(.*)&amp|isU',$doc,$file);
			if($file[1] != False){
				$ur = 'http://gegen-abzocke.com/xml/nstrim/fepcom/code.php?code_url='.$file[1];
				$html = HD::http_get_document($ur);
				$play = str_replace('md5hash',$html,$url[1]);
				$url = $play;
				}
			}
		elseif (preg_match("/kino-dom.tv/i",$media_url)){
			$doc = HD::http_get_document('http://kino-dom.tv/drama/1110-taynyy-krug-the-sesret-sirsle-1-sezon-1-seriya-eng-onlayn.html');
			preg_match('|pl:"http:\\/\\/kino-dom\\.tv\\/(.*)\\/play\\/|isU',$doc,$md5hash);
			$url = str_replace('md5hash',$md5hash[1],$media_url);
			}
		elseif (preg_match("/new-kino.net/i",$media_url)){
			$doc = HD::http_get_document('http://new-kino.net/komedii/5631-igrushka-1982.html');
			preg_match('|\\/dd11\\/(.*?)\\/|isU',$doc,$md5hash);
			$url = str_replace('md5hash',$md5hash[1],$media_url);
			}
		elseif (preg_match("/linecinema.org/i",$media_url)){
			$doc = HD::http_get_document('http://www.linecinema.org/newsz/boevyk-online/508954-bliznecy-drakony-twin-dragons-1992-dvdrip-onlayn.html');
			preg_match('|linecinema\\.org\\/s\\/(.*)\\/|isU',$doc,$md5hash);
			$url = str_replace('md5hash',$md5hash[1],$media_url);
			}
		elseif (preg_match("/kinostok.tv/i",$media_url)){
			$doc = HD::http_get_document('http://allserials.tv/get.php?action=playlist&pl=Osennie.cvety.2009');
			$doc = preg_replace('|\n|','',$doc);
			if($doc != False){
				$url = 'http://gegen-abzocke.com/xml/nstrim/kinostok/code.php?code_url='.$doc;
				$html = HD::http_get_document($url);
				preg_match('|file":"(.*)"|isU',$doc,$md5hash);
				$url = $md5hash[1];
				}
			}
		elseif (preg_match("/veterok.tv/i",$media_url)){
			$doc = HD::http_get_document($media_url);
			preg_match('|http://cdn(.*)"|isU',$doc,$md5hash);
			$url = 'http://cdn'.$md5hash[1];
			}
		elseif (preg_match("/uakino.net/i",$media_url)){
			$doc = HD::http_get_document($media_url);
			preg_match('|file":"(.*)"|isU',$doc,$md5hash);
			$url = $md5hash[1];
			}
		elseif (preg_match("/kinoprosmotr.net/i",$media_url)){
			$doc = file_get_contents($media_url);
			preg_match('|;file=(.*?)&amp;poster=|isU',$doc,$md5hash);
			$url = $md5hash[1].'.flv';
			}
		elseif(preg_match("|hdrezka.tv/films|i",$media_url)){
		$url = $media_url;
		$url = str_replace('http://hdrezka.tv/films@','',$url);
		header("Accept : text/plain, */*; q=0.01");
		header("Content-Type : application/x-www-form-urlencoded; charset=UTF-8");
		header("Host : hdrezka.tv");
		header("Referer : ".$url."");
		header("User-Agent : Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:27.0) Gecko/20100101 Firefox/27.0");
		header("X-Requested-With : XMLHttpRequest");
		preg_match('|http:\\/\\/.*\\/.*\\/.*\\/([0-9]*)-|isU',$url,$id);
		$data = array(
		  'id' => $id[1]);
		$postdata = http_build_query($data);
		$options = array('http' =>
		  array(
			'method' => 'POST',
			'header' => headers_list(),
			'content' => $postdata
		  )
		);
		$context = stream_context_create($options);
		$result = file_get_contents('http://hdrezka.tv/engine/ajax/getvideo.php', false, $context);
		$json = json_decode($result);
		$json2 = json_decode($json->link);
		$url = $json2->mp4;}
		elseif(preg_match("|hdrezka.tv/series|i",$media_url)){
		$url = $media_url;
		preg_match('|@(.*)@([0-9]*)@([0-9]*)@|isU',$url,$id);
		header("Accept : text/plain, */*; q=0.01");
		header("Content-Type : application/x-www-form-urlencoded; charset=UTF-8");
		header("Host : hdrezka.tv");
		header("Referer : ".$id[1]."");
		header("User-Agent : Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:27.0) Gecko/20100101 Firefox/27.0");
		header("X-Requested-With : XMLHttpRequest");
		preg_match('|http:\\/\\/.*\\/.*\\/.*\\/([0-9]*)-|isU',$url,$id2);
		$data = array(
		  'id' => $id2[1],
		  'season' => $id[3],
		  'episode' => $id[2]);
		$postdata = http_build_query($data);
		$options = array('http' =>
		  array(
			'method' => 'POST',
			'header' => headers_list(),
			'content' => $postdata
		  )
		);
		$context = stream_context_create($options);
		$result = file_get_contents('http://hdrezka.tv/engine/ajax/getvideo.php', false, $context);
		$json = json_decode($result);
		$json2 = json_decode($json->link);
		$url = $json2->mp4;}
		elseif(preg_match("|hdrezka.tv/gidtv|i",$media_url)){
		$url = $media_url;
		$url = str_replace('http://hdrezka.tv/gidtv@','',$url);
		$result = file_get_contents($url);
		preg_match_all('#http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+#',$result,$urls);
		for ($i=0; $i < count($urls[0]); $i++) {
		if(preg_match('|720.mp4/manifest.f4m|isU',$urls[0][$i])){
		$media_url = $urls[0][$i];
		$url = str_replace("',",'',$media_url);
		}
		elseif(preg_match('|480.mp4/manifest.f4m|isU',$urls[0][$i])){
			$media_url = $urls[0][$i];
			$url = str_replace("',",'',$media_url);}
				}
			}
		elseif (preg_match("/serialon.com/i",$media_url)){
		   $doc = HD::http_get_document('http://www.serialon.com/serial/10601-agent-osobogo-naznacheniya-4-15-seriya.html');
		   preg_match("|file':'http:\\/\\/www.serialon.com\\/.*\\/(.*)\\/.*\\/.*\\/.*.flv|isU",$doc,$md5hash);
		   $url = str_replace('md5hash',$md5hash[1],$media_url);
		   }
		elseif (preg_match("/serialsonline.net\/clip/i",$media_url)){
		   $doc = HD::http_get_document($media_url);
		   preg_match('|file:"(.*)"|isU',$doc,$md5hash);
		   $url = $md5hash[1];
		   }

		else
		{
			$url = urldecode($media_url);
		}
		
		$url = str_replace("http:///assets/videos/.vk.flv", "http://dl.dropboxusercontent.com/u/13455121/Dune/1.mp4", $url);
		
		if (preg_match("/flv/i",$url))
        {
			$url = 'http://ts://127.0.0.1:81/cgi-bin/flv.sh?'. $url;
		}
		elseif (preg_match("/rtmp:/i",$url))
        {
			$url = 'http://ts://127.0.0.1:81/cgi-bin/rtmp.sh?'. $url;
		}
		elseif (preg_match("/\/udp\//i",$url) || preg_match("/94.76.109.30/i",$url))
        {
			$url = str_replace("http://", "http://ts://", $url);
		}
		elseif (preg_match("/.mp4/i",$url))
        {	
		if (!preg_match("/http:\/\/mp4:\/\//i",$url))
			$url = str_replace("http://", "http://mp4://", $url);
		}
		if (preg_match("/.m3u8/i",$url))
        {	
		if (preg_match("/http:\/\/mp4:\/\//i",$url))
			$url = str_replace("http://mp4://", "http://", $url);
		}
		$url = str_replace("ftp://", "http://", $url);
		// hd_print("-->url:$url");
        return($url);
    }
    
    public function get_vod_info($media_url, &$plugin_cookies)
    {
        //-//hd_print("--> get_vod_info: started...");
 
        if ($media_url === 'dummy')
            return array();

        HD::decode_user_data($media_url, $media_str, $user_data);
        foreach ($user_data as $key => $value)
        //-//hd_print("  $key => $value");

        $season_ref = "";
        if (!HD::has_attribute($user_data, 'season_ref'))
        {
            if (!HD::has_attribute($user_data, 'video_id'))
                throw new Exception('No season_ref or video_id');
            
            $season_ref = $user_data->video_id;
        }
        else
        {
            $season_ref = $user_data->season_ref;
        }

	    $ndx_series = 0;
        if (HD::has_attribute($user_data, 'ndx_series'))
		    $ndx_series = $user_data->ndx_series;
            
        $title = '';
        if (HD::has_attribute($user_data, 'season_title'))
            $title = $user_data->season_title;

        $descr = '';
        $icon_path = '';
            
	    $series = array();
	    
        $series_urls = $this->parse_season($season_ref, $plugin_cookies);
        
        $video_quality = isset($plugin_cookies->video_quality) ? $plugin_cookies->video_quality : 1;

        $i = 0;
	    foreach ($series_urls as $s)
	    {
	        HD::decode_user_data($s, $media_str, $user_data);
            if ($i === 0)
            {
                if (HD::has_attribute($user_data, 'season_title'))
                    $title = $user_data->season_title;

                if (HD::has_attribute($user_data, 'description'))
                    $descr = $user_data->description;
                if (HD::has_attribute($user_data, 'poster_url'))
                    $icon_path = $user_data->poster_url;
            }
            
            $url = $user_data->episode_ref . "&season_ref=$season_ref";
			//$url = str_replace("http://", "http://mp4://", $user_data->episode_ref . "&season_ref=$season_ref");
		    
		    if (strlen($user_data->episode_ref) > 0)
		    {
			    array_push
			    (
				    $series,
		            array
        		    (
                		PluginVodSeriesInfo::name => $user_data->episode_title,
	                	PluginVodSeriesInfo::playback_url => $url,
		                PluginVodSeriesInfo::playback_url_is_stream_url => false
        		    )
			    );
                ++$i;
		    }
	    }
	
        $buf_time = isset($plugin_cookies->buf_time) ? $plugin_cookies->buf_time : 0;
         // // ////////-//hd_print('--->>> Buffer time: ' . $buf_time);

        return array
        (
            PluginVodInfo::name => $title,
            PluginVodInfo::description => $descr,
            PluginVodInfo::poster_url => $icon_path,
            PluginVodInfo::initial_series_ndx => $ndx_series,
            PluginVodInfo::buffering_ms => $buf_time,
            PluginVodInfo::series => $series
        );
    }
    public function get_vod_stream_url($media_url, &$plugin_cookies)
    {	$media_url = str_ireplace("&amp;", "&", $media_url);
          // // ////////-//hd_print("SPICE = $media_url");
		//-//hd_print("get_vod_stream_url: starting");

        $video_quality = isset($plugin_cookies->video_quality) ? $plugin_cookies->video_quality : 1;

        //$media_url = str_replace("mp4://", "", $media_url);
        
        $tmp = explode('&season_ref=', $media_url);

        $media_url = $tmp[0];
        $season_ref = $tmp[1];
		if (preg_match("/\.(?:mp3|wav|og(?:g|a)|flac|midi?|rm|m4a|aac|wma|mka|ape|m3u8)$/i",$media_url) || preg_match("/\/udp\//i",$media_url))
				{
					return $this->get_episode_url($media_url, $video_quality);				
				}
		else
		{
        $links = $media_url;
        
        	//{ Viewed items
			$s = $season_ref . ".viewed";
			if (isset($plugin_cookies->$s))
			{
				if (!ListUtil::is_in_list($plugin_cookies->$s, $media_url))
				{
					$i = $plugin_cookies->$s;
					$plugin_cookies->$s = ListUtil::add_item($i, $media_url);
				}
			}
			else                       
			{
				$plugin_cookies->$s = $media_url;
			}
			//} Viewed items
		

        return $this->get_episode_url($media_url, $video_quality);
		}
    }

    ///////////////////////////////////////////////////////////////////////

    private function get_loading_items()
    {
        $view_type = isset($plugin_cookies->view_type) ? $plugin_cookies->view_type : 2;
        $items = array(
            array
            (
                PluginRegularFolderItem::caption =>
                    (is_interface_language_russian_or_similar() ? 'Загрузка...' : 'Loading...'),
                PluginRegularFolderItem::view_item_params => array(
                    ViewItemParams::item_paint_icon => true,
                    ViewItemParams::item_layout => HALIGN_LEFT,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::icon_dx => 10,
                    ViewItemParams::icon_width => ($view_type == 1 ? 60 : ($view_type == 2 ? 150 : 50)),
                    ViewItemParams::icon_height => (1 ? 60 : ($view_type == 2 ? 150 : 50)),
                    ViewItemParams::icon_path => ($view_type != 3 ? 'gui_skin://large_icons/movie.aai' : 'gui_skin://small_icons/movie.aai'),
                    ViewItemParams::item_caption_width => 950,
                    ViewItemParams::item_detailed_icon_path => 'missing://'
                ),
                PluginRegularFolderItem::media_url => 'dummy'
            )
        );

        return array(
            PluginRegularFolderRange::total => count($items),
            PluginRegularFolderRange::count => count($items),
            PluginRegularFolderRange::more_items_available => 1,
            PluginRegularFolderRange::from_ndx => 0,
            PluginRegularFolderRange::items => $items
        );
    }

    ///////////////////////////////////////////////////////////////////////
    private function cScVOD_main($media_url, &$plugin_cookies)
    {
		$api = HOST_API_URL;
        $url = $api . '/vod/start.xml';
        $doc = HD::http_get_document($url);
        // get list of categories
       $tmp = explode('</logo_30x30>', $doc);
		$main_menu_block = strstr($tmp[1], '<title>Наши плейлисты', true);
        
      // ////////-//hd_print("--->>> main_menu_block: $main_menu_block");
        
		$videos = explode('<title>', $main_menu_block);
		unset($videos[0]);
		$videos = array_values($videos);

        $items = array();
    

        foreach($videos as $video)
        {
			$category_ref = explode('<playlist_url><![CDATA[', $video);
			$category_ref = strstr($category_ref[1], ']', true);
			$category_name = strstr($video, '</title>', true);
            if (preg_match("/vk2.php/i",$category_ref)){
			array_push
            (
                $items,
                array
                (
                    PluginRegularFolderItem::caption => $category_name,
                    PluginRegularFolderItem::view_item_params => self::$catalog_vip,
                    PluginRegularFolderItem::media_url =>
                            HD::encode_user_data
                            (
                                array
                                (
                                    'search_ref' => $category_ref,
                                )
                            )
                )
            );
			}
			else{
			array_push
            (
                $items,
                array
                (
                    PluginRegularFolderItem::caption => $category_name,
                    PluginRegularFolderItem::view_item_params => self::$catalog_vip,
                    PluginRegularFolderItem::media_url =>
                            HD::encode_user_data
                            (
                                array
                                (
                                    'category_ref' => $category_ref,
                                )
                            )
                )
            );}
        }        
        return $items;
    }
    ///////////////////////////////////////////////////////////////////////

    private function get_main_menu_items($media_url, &$plugin_cookies)
    {
        HD::decode_user_data($media_url,$url,$user_data);

        if (update_interface_language())
            FolderCache::clear();

        $skin = isset($plugin_cookies->skin) ? $plugin_cookies->skin : 'default';
	    if ($media_url === 'main_menu')
	    {

		    $items = array
		    (
			    array
			    (
				    PluginRegularFolderItem::caption => 'Категории',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/categories.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'main_menu:videos'
                ),
			    array
			    (
				    PluginRegularFolderItem::caption => 'Поиск',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/search.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'main_menu:search'
                ),
			    array
			    (
				    PluginRegularFolderItem::caption => 'Избранное',
				    PluginRegularFolderItem::view_item_params => array(
				            ViewItemParams::icon_path => self::get_skin_path($plugin_cookies) . 'icons/favorites.png',
				            ViewItemParams::item_layout => HALIGN_CENTER,
				            ViewItemParams::icon_valign => VALIGN_CENTER,
					        ViewItemParams::item_paint_caption_within_icon => false,
                            ViewItemParams::item_paint_caption => ($skin == 'juniperus' ? true : false),
				            ViewItemParams::item_caption_font_size => FONT_SIZE_NORMAL,
				    ),
				    PluginRegularFolderItem::media_url => 'main_menu:fav'
                ),
		    );
	    }
        else if ($media_url === 'main_menu:search')
	    {
		    $items = array();
            array_push
            (
                $items,
                array
                (
                    PluginRegularFolderItem::caption => '[Новый запрос]',
                    PluginRegularFolderItem::view_item_params => self::$catalog_vip,
                    PluginRegularFolderItem::media_url =>
                        HD::encode_user_data
                        (
                            array('ser_feed' => '')
                        )
                )
            );

		    $search_items = isset($plugin_cookies->search_items) ? $plugin_cookies->search_items : '';
		    foreach (ListUtil::string_to_list($search_items) as $item)
		    {
			    if ($item !== "")
			    {
				    array_push
				    (
					    $items,
					    array
					    (
						    PluginRegularFolderItem::caption => (is_interface_language_russian_or_similar() ? "поиск: $item" : "search: $item"),
						    PluginRegularFolderItem::view_item_params => self::$catalog_vip,
						    PluginRegularFolderItem::media_url =>
							    HD::encode_user_data
							    (
								    array('ser_feed' => urlencode($item))
							    )
                        )
                    );
                }
            }
        }
		else if (HD::has_attribute($user_data, 'search_ref'))
	    {	$search_link = $user_data->search_ref;
		    $items = array();
            array_push
            (
                $items,
                array
                (
                    PluginRegularFolderItem::caption => '[Новый запрос]',
                    PluginRegularFolderItem::view_item_params => self::$catalog_vip,
                    PluginRegularFolderItem::media_url =>
                        HD::encode_user_data
                        (
                            array
							(
							'ser_feed' => ''
							)
                        )
                )
            );

		    $search_items = isset($plugin_cookies->search_items) ? $plugin_cookies->search_items : '';
		    foreach (ListUtil::string_to_list($search_items) as $item)
		    {	if (preg_match("/p=1/i",$search_link)){
				$search_text = $search_link . '&search=' . urlencode($item);}
				else{
			    $search_text = $search_link . '?search=' . urlencode($item);}
				if ($item !== "")
			    {
				    array_push
				    (
					    $items,
					    array
					    (
						    PluginRegularFolderItem::caption => (is_interface_language_russian_or_similar() ? "поиск: $item" : "search: $item"),
						    PluginRegularFolderItem::view_item_params => self::$catalog_vip,
						    PluginRegularFolderItem::media_url =>
							    HD::encode_user_data
							    (
								    array('ser_feed' => $search_text)
							    )
                        )
                    );
                }
            }
        }
        else if ($media_url === 'main_menu:fav')
	    {
		    $items = array();
		    $fav_items = isset($plugin_cookies->fav_items) ? $plugin_cookies->fav_items : '';
		    foreach (ListUtil::string_to_list($fav_items) as $item)
		    {
			    if ($item !== "")
			    {
				    $folderRange = $this->get_regular_folder_items(
							    HD::encode_user_data(array('fav' => $item)),
							    0,
							    $plugin_cookies
					   	    );

				    if (!$folderRange)
					    continue;
				    if ($folderRange[PluginRegularFolderRange::count] < 1)
					    continue;

				    array_push
				    (
					    $items,
					    $folderRange[PluginRegularFolderRange::items][0]
                    );
                }
            }
        }
	    elseif ($media_url === 'main_menu:videos')
	    {
            $items = $this->cScVOD_main($media_url, $plugin_cookies);
	    }
	    return 	array
	    (
		        PluginRegularFolderRange::total => count($items),
		        PluginRegularFolderRange::count => count($items),
	            PluginRegularFolderRange::more_items_available => false,
		        PluginRegularFolderRange::from_ndx => 0,
		        PluginRegularFolderRange::items => $items
	    );
	}
}

///////////////////////////////////////////////////////////////////////////

DefaultDunePluginFw::$plugin_class_name = 'cScVODPlugin';

///////////////////////////////////////////////////////////////////////////
?>
